from fastapi import Depends, FastAPI
from fastapi.security import OAuth2PasswordBearer
from typing_extensions import Annotated

app = FastAPI()

oauth2_scheme = OAuth2PasswordBearer(tokenUrl="token")


@app.get("/items/")
async def read_items(token: Annotated[str, Depends(oauth2_scheme)]):
    return {"token": token}



from fastapi.testclient import TestClient

from docs_src.behind_a_proxy.tutorial001_01 import app

client = TestClient(
    app,
    base_url="https://example.com",
    follow_redirects=False,
)


def test_redirect() -> None:
    response = client.get("/items")
    assert response.status_code == 307
    assert response.headers["location"] == "https://example.com/items/"


def test_no_redirect() -> None:
    response = client.get("/items/")
    assert response.status_code == 200
    assert response.json() == ["plumbus", "portal gun"]



from typing import Optional

from fastapi import FastAPI
from fastapi.params import Param
from fastapi.testclient import TestClient

app = FastAPI()


@app.get("/items/")
def read_items(q: Optional[str] = Param(default=None)):  # type: ignore
    return {"q": q}


client = TestClient(app)


def test_default_param_query_none():
    response = client.get("/items/")
    assert response.status_code == 200, response.text
    assert response.json() == {"q": None}


def test_default_param_query():
    response = client.get("/items/?q=foo")
    assert response.status_code == 200, response.text
    assert response.json() == {"q": "foo"}



import sys
from typing import List, Optional

import pytest

from tests.utils import pydantic_snapshot, skip_module_if_py_gte_314

if sys.version_info >= (3, 14):
    skip_module_if_py_gte_314()

from fastapi import FastAPI
from fastapi._compat.v1 import BaseModel
from fastapi.temp_pydantic_v1_params import (
    Body,
    Cookie,
    File,
    Form,
    Header,
    Path,
    Query,
)
from fastapi.testclient import TestClient
from inline_snapshot import snapshot
from typing_extensions import Annotated


class Item(BaseModel):
    name: str
    price: float
    description: Optional[str] = None


app = FastAPI()


@app.get("/items/{item_id}")
def get_item_with_path(
    item_id: Annotated[int, Path(title="The ID of the item", ge=1, le=1000)],
):
    return {"item_id": item_id}


@app.get("/items/")
def get_items_with_query(
    q: Annotated[
        Optional[str], Query(min_length=3, max_length=50, pattern="^[a-zA-Z0-9 ]+$")
    ] = None,
    skip: Annotated[int, Query(ge=0)] = 0,
    limit: Annotated[int, Query(ge=1, le=100, examples=[5])] = 10,
):
    return {"q": q, "skip": skip, "limit": limit}


@app.get("/users/")
def get_user_with_header(
    x_custom: Annotated[Optional[str], Header()] = None,
    x_token: Annotated[Optional[str], Header(convert_underscores=True)] = None,
):
    return {"x_custom": x_custom, "x_token": x_token}


@app.get("/cookies/")
def get_cookies(
    session_id: Annotated[Optional[str], Cookie()] = None,
    tracking_id: Annotated[Optional[str], Cookie(min_length=10)] = None,
):
    return {"session_id": session_id, "tracking_id": tracking_id}


@app.post("/items/")
def create_item(
    item: Annotated[
        Item,
        Body(examples=[{"name": "Foo", "price": 35.4, "description": "The Foo item"}]),
    ],
):
    return {"item": item}


@app.post("/items-embed/")
def create_item_embed(
    item: Annotated[Item, Body(embed=True)],
):
    return {"item": item}


@app.put("/items/{item_id}")
def update_item(
    item_id: Annotated[int, Path(ge=1)],
    item: Annotated[Item, Body()],
    importance: Annotated[int, Body(gt=0, le=10)],
):
    return {"item": item, "importance": importance}


@app.post("/form-data/")
def submit_form(
    username: Annotated[str, Form(min_length=3, max_length=50)],
    password: Annotated[str, Form(min_length=8)],
    email: Annotated[Optional[str], Form()] = None,
):
    return {"username": username, "password": password, "email": email}


@app.post("/upload/")
def upload_file(
    file: Annotated[bytes, File()],
    description: Annotated[Optional[str], Form()] = None,
):
    return {"file_size": len(file), "description": description}


@app.post("/upload-multiple/")
def upload_multiple_files(
    files: Annotated[List[bytes], File()],
    note: Annotated[str, Form()] = "",
):
    return {
        "file_count": len(files),
        "total_size": sum(len(f) for f in files),
        "note": note,
    }


client = TestClient(app)


# Path parameter tests
def test_path_param_valid():
    response = client.get("/items/50")
    assert response.status_code == 200
    assert response.json() == {"item_id": 50}


def test_path_param_too_large():
    response = client.get("/items/1001")
    assert response.status_code == 422
    error = response.json()["detail"][0]
    assert error["loc"] == ["path", "item_id"]


def test_path_param_too_small():
    response = client.get("/items/0")
    assert response.status_code == 422
    error = response.json()["detail"][0]
    assert error["loc"] == ["path", "item_id"]


# Query parameter tests
def test_query_params_valid():
    response = client.get("/items/?q=test search&skip=5&limit=20")
    assert response.status_code == 200
    assert response.json() == {"q": "test search", "skip": 5, "limit": 20}


def test_query_params_defaults():
    response = client.get("/items/")
    assert response.status_code == 200
    assert response.json() == {"q": None, "skip": 0, "limit": 10}


def test_query_param_too_short():
    response = client.get("/items/?q=ab")
    assert response.status_code == 422
    error = response.json()["detail"][0]
    assert error["loc"] == ["query", "q"]


def test_query_param_invalid_pattern():
    response = client.get("/items/?q=test@#$")
    assert response.status_code == 422
    error = response.json()["detail"][0]
    assert error["loc"] == ["query", "q"]


def test_query_param_limit_too_large():
    response = client.get("/items/?limit=101")
    assert response.status_code == 422
    error = response.json()["detail"][0]
    assert error["loc"] == ["query", "limit"]


# Header parameter tests
def test_header_params():
    response = client.get(
        "/users/",
        headers={"X-Custom": "Plumbus", "X-Token": "secret-token"},
    )
    assert response.status_code == 200
    assert response.json() == {
        "x_custom": "Plumbus",
        "x_token": "secret-token",
    }


def test_header_underscore_conversion():
    response = client.get(
        "/users/",
        headers={"x-token": "secret-token-with-dash"},
    )
    assert response.status_code == 200
    assert response.json()["x_token"] == "secret-token-with-dash"


def test_header_params_none():
    response = client.get("/users/")
    assert response.status_code == 200
    assert response.json() == {"x_custom": None, "x_token": None}


# Cookie parameter tests
def test_cookie_params():
    with TestClient(app) as client:
        client.cookies.set("session_id", "abc123")
        client.cookies.set("tracking_id", "1234567890abcdef")
        response = client.get("/cookies/")
    assert response.status_code == 200
    assert response.json() == {
        "session_id": "abc123",
        "tracking_id": "1234567890abcdef",
    }


def test_cookie_tracking_id_too_short():
    with TestClient(app) as client:
        client.cookies.set("tracking_id", "short")
        response = client.get("/cookies/")
    assert response.status_code == 422
    assert response.json() == snapshot(
        {
            "detail": [
                {
                    "loc": ["cookie", "tracking_id"],
                    "msg": "ensure this value has at least 10 characters",
                    "type": "value_error.any_str.min_length",
                    "ctx": {"limit_value": 10},
                }
            ]
        }
    )


def test_cookie_params_none():
    response = client.get("/cookies/")
    assert response.status_code == 200
    assert response.json() == {"session_id": None, "tracking_id": None}


# Body parameter tests
def test_body_param():
    response = client.post(
        "/items/",
        json={"name": "Test Item", "price": 29.99, "description": "A test item"},
    )
    assert response.status_code == 200
    assert response.json() == {
        "item": {
            "name": "Test Item",
            "price": 29.99,
            "description": "A test item",
        }
    }


def test_body_param_minimal():
    response = client.post(
        "/items/",
        json={"name": "Minimal", "price": 9.99},
    )
    assert response.status_code == 200
    assert response.json() == {
        "item": {"name": "Minimal", "price": 9.99, "description": None}
    }


def test_body_param_missing_required():
    response = client.post(
        "/items/",
        json={"name": "Incomplete"},
    )
    assert response.status_code == 422
    error = response.json()["detail"][0]
    assert error["loc"] == ["body", "price"]


def test_body_embed():
    response = client.post(
        "/items-embed/",
        json={"item": {"name": "Embedded", "price": 15.0}},
    )
    assert response.status_code == 200
    assert response.json() == {
        "item": {"name": "Embedded", "price": 15.0, "description": None}
    }


def test_body_embed_wrong_structure():
    response = client.post(
        "/items-embed/",
        json={"name": "Not Embedded", "price": 15.0},
    )
    assert response.status_code == 422


# Multiple body parameters test
def test_multiple_body_params():
    response = client.put(
        "/items/5",
        json={
            "item": {"name": "Updated Item", "price": 49.99},
            "importance": 8,
        },
    )
    assert response.status_code == 200
    assert response.json() == snapshot(
        {
            "item": {"name": "Updated Item", "price": 49.99, "description": None},
            "importance": 8,
        }
    )


def test_multiple_body_params_importance_too_large():
    response = client.put(
        "/items/5",
        json={
            "item": {"name": "Item", "price": 10.0},
            "importance": 11,
        },
    )
    assert response.status_code == 422
    assert response.json() == snapshot(
        {
            "detail": [
                {
                    "loc": ["body", "importance"],
                    "msg": "ensure this value is less than or equal to 10",
                    "type": "value_error.number.not_le",
                    "ctx": {"limit_value": 10},
                }
            ]
        }
    )


def test_multiple_body_params_importance_too_small():
    response = client.put(
        "/items/5",
        json={
            "item": {"name": "Item", "price": 10.0},
            "importance": 0,
        },
    )
    assert response.status_code == 422
    assert response.json() == snapshot(
        {
            "detail": [
                {
                    "loc": ["body", "importance"],
                    "msg": "ensure this value is greater than 0",
                    "type": "value_error.number.not_gt",
                    "ctx": {"limit_value": 0},
                }
            ]
        }
    )


# Form parameter tests
def test_form_data_valid():
    response = client.post(
        "/form-data/",
        data={
            "username": "testuser",
            "password": "password123",
            "email": "test@example.com",
        },
    )
    assert response.status_code == 200, response.text
    assert response.json() == {
        "username": "testuser",
        "password": "password123",
        "email": "test@example.com",
    }


def test_form_data_optional_field():
    response = client.post(
        "/form-data/",
        data={"username": "testuser", "password": "password123"},
    )
    assert response.status_code == 200
    assert response.json() == {
        "username": "testuser",
        "password": "password123",
        "email": None,
    }


def test_form_data_username_too_short():
    response = client.post(
        "/form-data/",
        data={"username": "ab", "password": "password123"},
    )
    assert response.status_code == 422
    assert response.json() == snapshot(
        {
            "detail": [
                {
                    "loc": ["body", "username"],
                    "msg": "ensure this value has at least 3 characters",
                    "type": "value_error.any_str.min_length",
                    "ctx": {"limit_value": 3},
                }
            ]
        }
    )


def test_form_data_password_too_short():
    response = client.post(
        "/form-data/",
        data={"username": "testuser", "password": "short"},
    )
    assert response.status_code == 422
    assert response.json() == snapshot(
        {
            "detail": [
                {
                    "loc": ["body", "password"],
                    "msg": "ensure this value has at least 8 characters",
                    "type": "value_error.any_str.min_length",
                    "ctx": {"limit_value": 8},
                }
            ]
        }
    )


# File upload tests
def test_upload_file():
    response = client.post(
        "/upload/",
        files={"file": ("test.txt", b"Hello, World!", "text/plain")},
        data={"description": "A test file"},
    )
    assert response.status_code == 200
    assert response.json() == {
        "file_size": 13,
        "description": "A test file",
    }


def test_upload_file_without_description():
    response = client.post(
        "/upload/",
        files={"file": ("test.txt", b"Hello!", "text/plain")},
    )
    assert response.status_code == 200
    assert response.json() == {
        "file_size": 6,
        "description": None,
    }


def test_upload_multiple_files():
    response = client.post(
        "/upload-multiple/",
        files=[
            ("files", ("file1.txt", b"Content 1", "text/plain")),
            ("files", ("file2.txt", b"Content 2", "text/plain")),
            ("files", ("file3.txt", b"Content 3", "text/plain")),
        ],
        data={"note": "Multiple files uploaded"},
    )
    assert response.status_code == 200
    assert response.json() == {
        "file_count": 3,
        "total_size": 27,
        "note": "Multiple files uploaded",
    }


def test_upload_multiple_files_empty_note():
    response = client.post(
        "/upload-multiple/",
        files=[
            ("files", ("file1.txt", b"Test", "text/plain")),
        ],
    )
    assert response.status_code == 200
    assert response.json()["file_count"] == 1
    assert response.json()["note"] == ""


# __repr__ tests
def test_query_repr():
    query_param = Query(default=None, min_length=3)
    assert repr(query_param) == "Query(None)"


def test_body_repr():
    body_param = Body(default=None)
    assert repr(body_param) == "Body(None)"


# Deprecation warning tests for regex parameter
def test_query_regex_deprecation_warning():
    with pytest.warns(DeprecationWarning, match="`regex` has been deprecated"):
        Query(regex="^test$")


def test_body_regex_deprecation_warning():
    with pytest.warns(DeprecationWarning, match="`regex` has been deprecated"):
        Body(regex="^test$")


# Deprecation warning tests for example parameter
def test_query_example_deprecation_warning():
    with pytest.warns(DeprecationWarning, match="`example` has been deprecated"):
        Query(example="test example")


def test_body_example_deprecation_warning():
    with pytest.warns(DeprecationWarning, match="`example` has been deprecated"):
        Body(example={"test": "example"})


def test_openapi_schema():
    response = client.get("/openapi.json")
    assert response.status_code == 200, response.text
    assert response.json() == snapshot(
        {
            "openapi": "3.1.0",
            "info": {"title": "FastAPI", "version": "0.1.0"},
            "paths": {
                "/items/{item_id}": {
                    "get": {
                        "summary": "Get Item With Path",
                        "operationId": "get_item_with_path_items__item_id__get",
                        "parameters": [
                            {
                                "name": "item_id",
                                "in": "path",
                                "required": True,
                                "schema": {
                                    "title": "The ID of the item",
                                    "minimum": 1,
                                    "maximum": 1000,
                                    "type": "integer",
                                },
                            }
                        ],
                        "responses": {
                            "200": {
                                "description": "Successful Response",
                                "content": {"application/json": {"schema": {}}},
                            },
                            "422": {
                                "description": "Validation Error",
                                "content": {
                                    "application/json": {
                                        "schema": {
                                            "$ref": "#/components/schemas/HTTPValidationError"
                                        }
                                    }
                                },
                            },
                        },
                    },
                    "put": {
                        "summary": "Update Item",
                        "operationId": "update_item_items__item_id__put",
                        "parameters": [
                            {
                                "name": "item_id",
                                "in": "path",
                                "required": True,
                                "schema": {
                                    "title": "Item Id",
                                    "minimum": 1,
                                    "type": "integer",
                                },
                            }
                        ],
                        "requestBody": {
                            "required": True,
                            "content": {
                                "application/json": {
                                    "schema": pydantic_snapshot(
                                        v1=snapshot(
                                            {
                                                "$ref": "#/components/schemas/Body_update_item_items__item_id__put"
                                            }
                                        ),
                                        v2=snapshot(
                                            {
                                                "title": "Body",
                                                "allOf": [
                                                    {
                                                        "$ref": "#/components/schemas/Body_update_item_items__item_id__put"
                                                    }
                                                ],
                                            }
                                        ),
                                    ),
                                }
                            },
                        },
                        "responses": {
                            "200": {
                                "description": "Successful Response",
                                "content": {"application/json": {"schema": {}}},
                            },
                            "422": {
                                "description": "Validation Error",
                                "content": {
                                    "application/json": {
                                        "schema": {
                                            "$ref": "#/components/schemas/HTTPValidationError"
                                        }
                                    }
                                },
                            },
                        },
                    },
                },
                "/items/": {
                    "get": {
                        "summary": "Get Items With Query",
                        "operationId": "get_items_with_query_items__get",
                        "parameters": [
                            {
                                "name": "q",
                                "in": "query",
                                "required": False,
                                "schema": {
                                    "title": "Q",
                                    "maxLength": 50,
                                    "minLength": 3,
                                    "pattern": "^[a-zA-Z0-9 ]+$",
                                    "type": "string",
                                },
                            },
                            {
                                "name": "skip",
                                "in": "query",
                                "required": False,
                                "schema": {
                                    "title": "Skip",
                                    "default": 0,
                                    "minimum": 0,
                                    "type": "integer",
                                },
                            },
                            {
                                "name": "limit",
                                "in": "query",
                                "required": False,
                                "schema": {
                                    "title": "Limit",
                                    "default": 10,
                                    "minimum": 1,
                                    "maximum": 100,
                                    "examples": [5],
                                    "type": "integer",
                                },
                            },
                        ],
                        "responses": {
                            "200": {
                                "description": "Successful Response",
                                "content": {"application/json": {"schema": {}}},
                            },
                            "422": {
                                "description": "Validation Error",
                                "content": {
                                    "application/json": {
                                        "schema": {
                                            "$ref": "#/components/schemas/HTTPValidationError"
                                        }
                                    }
                                },
                            },
                        },
                    },
                    "post": {
                        "summary": "Create Item",
                        "operationId": "create_item_items__post",
                        "requestBody": {
                            "required": True,
                            "content": {
                                "application/json": {
                                    "schema": {
                                        "title": "Item",
                                        "examples": [
                                            {
                                                "name": "Foo",
                                                "price": 35.4,
                                                "description": "The Foo item",
                                            }
                                        ],
                                        "allOf": [
                                            {"$ref": "#/components/schemas/Item"}
                                        ],
                                    }
                                }
                            },
                        },
                        "responses": {
                            "200": {
                                "description": "Successful Response",
                                "content": {"application/json": {"schema": {}}},
                            },
                            "422": {
                                "description": "Validation Error",
                                "content": {
                                    "application/json": {
                                        "schema": {
                                            "$ref": "#/components/schemas/HTTPValidationError"
                                        }
                                    }
                                },
                            },
                        },
                    },
                },
                "/users/": {
                    "get": {
                        "summary": "Get User With Header",
                        "operationId": "get_user_with_header_users__get",
                        "parameters": [
                            {
                                "name": "x-custom",
                                "in": "header",
                                "required": False,
                                "schema": {"title": "X-Custom", "type": "string"},
                            },
                            {
                                "name": "x-token",
                                "in": "header",
                                "required": False,
                                "schema": {"title": "X-Token", "type": "string"},
                            },
                        ],
                        "responses": {
                            "200": {
                                "description": "Successful Response",
                                "content": {"application/json": {"schema": {}}},
                            },
                            "422": {
                                "description": "Validation Error",
                                "content": {
                                    "application/json": {
                                        "schema": {
                                            "$ref": "#/components/schemas/HTTPValidationError"
                                        }
                                    }
                                },
                            },
                        },
                    }
                },
                "/cookies/": {
                    "get": {
                        "summary": "Get Cookies",
                        "operationId": "get_cookies_cookies__get",
                        "parameters": [
                            {
                                "name": "session_id",
                                "in": "cookie",
                                "required": False,
                                "schema": {"title": "Session Id", "type": "string"},
                            },
                            {
                                "name": "tracking_id",
                                "in": "cookie",
                                "required": False,
                                "schema": {
                                    "title": "Tracking Id",
                                    "minLength": 10,
                                    "type": "string",
                                },
                            },
                        ],
                        "responses": {
                            "200": {
                                "description": "Successful Response",
                                "content": {"application/json": {"schema": {}}},
                            },
                            "422": {
                                "description": "Validation Error",
                                "content": {
                                    "application/json": {
                                        "schema": {
                                            "$ref": "#/components/schemas/HTTPValidationError"
                                        }
                                    }
                                },
                            },
                        },
                    }
                },
                "/items-embed/": {
                    "post": {
                        "summary": "Create Item Embed",
                        "operationId": "create_item_embed_items_embed__post",
                        "requestBody": {
                            "content": {
                                "application/json": {
                                    "schema": pydantic_snapshot(
                                        v1=snapshot(
                                            {
                                                "$ref": "#/components/schemas/Body_create_item_embed_items_embed__post"
                                            }
                                        ),
                                        v2=snapshot(
                                            {
                                                "allOf": [
                                                    {
                                                        "$ref": "#/components/schemas/Body_create_item_embed_items_embed__post"
                                                    }
                                                ],
                                                "title": "Body",
                                            }
                                        ),
                                    ),
                                }
                            },
                            "required": True,
                        },
                        "responses": {
                            "200": {
                                "description": "Successful Response",
                                "content": {"application/json": {"schema": {}}},
                            },
                            "422": {
                                "description": "Validation Error",
                                "content": {
                                    "application/json": {
                                        "schema": {
                                            "$ref": "#/components/schemas/HTTPValidationError"
                                        }
                                    }
                                },
                            },
                        },
                    }
                },
                "/form-data/": {
                    "post": {
                        "summary": "Submit Form",
                        "operationId": "submit_form_form_data__post",
                        "requestBody": {
                            "content": {
                                "application/x-www-form-urlencoded": {
                                    "schema": pydantic_snapshot(
                                        v1=snapshot(
                                            {
                                                "$ref": "#/components/schemas/Body_submit_form_form_data__post"
                                            }
                                        ),
                                        v2=snapshot(
                                            {
                                                "allOf": [
                                                    {
                                                        "$ref": "#/components/schemas/Body_submit_form_form_data__post"
                                                    }
                                                ],
                                                "title": "Body",
                                            }
                                        ),
                                    ),
                                }
                            },
                            "required": True,
                        },
                        "responses": {
                            "200": {
                                "description": "Successful Response",
                                "content": {"application/json": {"schema": {}}},
                            },
                            "422": {
                                "description": "Validation Error",
                                "content": {
                                    "application/json": {
                                        "schema": {
                                            "$ref": "#/components/schemas/HTTPValidationError"
                                        }
                                    }
                                },
                            },
                        },
                    }
                },
                "/upload/": {
                    "post": {
                        "summary": "Upload File",
                        "operationId": "upload_file_upload__post",
                        "requestBody": {
                            "content": {
                                "multipart/form-data": {
                                    "schema": pydantic_snapshot(
                                        v1=snapshot(
                                            {
                                                "$ref": "#/components/schemas/Body_upload_file_upload__post"
                                            }
                                        ),
                                        v2=snapshot(
                                            {
                                                "allOf": [
                                                    {
                                                        "$ref": "#/components/schemas/Body_upload_file_upload__post"
                                                    }
                                                ],
                                                "title": "Body",
                                            }
                                        ),
                                    ),
                                }
                            },
                            "required": True,
                        },
                        "responses": {
                            "200": {
                                "description": "Successful Response",
                                "content": {"application/json": {"schema": {}}},
                            },
                            "422": {
                                "description": "Validation Error",
                                "content": {
                                    "application/json": {
                                        "schema": {
                                            "$ref": "#/components/schemas/HTTPValidationError"
                                        }
                                    }
                                },
                            },
                        },
                    }
                },
                "/upload-multiple/": {
                    "post": {
                        "summary": "Upload Multiple Files",
                        "operationId": "upload_multiple_files_upload_multiple__post",
                        "requestBody": {
                            "content": {
                                "multipart/form-data": {
                                    "schema": pydantic_snapshot(
                                        v1=snapshot(
                                            {
                                                "$ref": "#/components/schemas/Body_upload_multiple_files_upload_multiple__post"
                                            }
                                        ),
                                        v2=snapshot(
                                            {
                                                "allOf": [
                                                    {
                                                        "$ref": "#/components/schemas/Body_upload_multiple_files_upload_multiple__post"
                                                    }
                                                ],
                                                "title": "Body",
                                            }
                                        ),
                                    ),
                                }
                            },
                            "required": True,
                        },
                        "responses": {
                            "200": {
                                "description": "Successful Response",
                                "content": {"application/json": {"schema": {}}},
                            },
                            "422": {
                                "description": "Validation Error",
                                "content": {
                                    "application/json": {
                                        "schema": {
                                            "$ref": "#/components/schemas/HTTPValidationError"
                                        }
                                    }
                                },
                            },
                        },
                    }
                },
            },
            "components": {
                "schemas": {
                    "Body_create_item_embed_items_embed__post": {
                        "properties": pydantic_snapshot(
                            v1=snapshot(
                                {"item": {"$ref": "#/components/schemas/Item"}}
                            ),
                            v2=snapshot(
                                {
                                    "item": {
                                        "allOf": [
                                            {"$ref": "#/components/schemas/Item"}
                                        ],
                                        "title": "Item",
                                    }
                                }
                            ),
                        ),
                        "type": "object",
                        "required": ["item"],
                        "title": "Body_create_item_embed_items_embed__post",
                    },
                    "Body_submit_form_form_data__post": {
                        "properties": {
                            "username": {
                                "type": "string",
                                "maxLength": 50,
                                "minLength": 3,
                                "title": "Username",
                            },
                            "password": {
                                "type": "string",
                                "minLength": 8,
                                "title": "Password",
                            },
                            "email": {"type": "string", "title": "Email"},
                        },
                        "type": "object",
                        "required": ["username", "password"],
                        "title": "Body_submit_form_form_data__post",
                    },
                    "Body_update_item_items__item_id__put": {
                        "properties": {
                            "item": pydantic_snapshot(
                                v1=snapshot({"$ref": "#/components/schemas/Item"}),
                                v2=snapshot(
                                    {
                                        "allOf": [
                                            {"$ref": "#/components/schemas/Item"}
                                        ],
                                        "title": "Item",
                                    }
                                ),
                            ),
                            "importance": {
                                "type": "integer",
                                "maximum": 10.0,
                                "exclusiveMinimum": 0.0,
                                "title": "Importance",
                            },
                        },
                        "type": "object",
                        "required": ["item", "importance"],
                        "title": "Body_update_item_items__item_id__put",
                    },
                    "Body_upload_file_upload__post": {
                        "properties": {
                            "file": {
                                "type": "string",
                                "format": "binary",
                                "title": "File",
                            },
                            "description": {"type": "string", "title": "Description"},
                        },
                        "type": "object",
                        "required": ["file"],
                        "title": "Body_upload_file_upload__post",
                    },
                    "Body_upload_multiple_files_upload_multiple__post": {
                        "properties": {
                            "files": {
                                "items": {"type": "string", "format": "binary"},
                                "type": "array",
                                "title": "Files",
                            },
                            "note": {"type": "string", "title": "Note", "default": ""},
                        },
                        "type": "object",
                        "required": ["files"],
                        "title": "Body_upload_multiple_files_upload_multiple__post",
                    },
                    "HTTPValidationError": {
                        "properties": {
                            "detail": {
                                "items": {
                                    "$ref": "#/components/schemas/ValidationError"
                                },
                                "type": "array",
                                "title": "Detail",
                            }
                        },
                        "type": "object",
                        "title": "HTTPValidationError",
                    },
                    "Item": {
                        "properties": {
                            "name": {"type": "string", "title": "Name"},
                            "price": {"type": "number", "title": "Price"},
                            "description": {"type": "string", "title": "Description"},
                        },
                        "type": "object",
                        "required": ["name", "price"],
                        "title": "Item",
                    },
                    "ValidationError": {
                        "properties": {
                            "loc": {
                                "items": {
                                    "anyOf": [{"type": "string"}, {"type": "integer"}]
                                },
                                "type": "array",
                                "title": "Location",
                            },
                            "msg": {"type": "string", "title": "Message"},
                            "type": {"type": "string", "title": "Error Type"},
                        },
                        "type": "object",
                        "required": ["loc", "msg", "type"],
                        "title": "ValidationError",
                    },
                }
            },
        }
    )



import importlib

import pytest
from dirty_equals import IsDict, IsOneOf
from fastapi.testclient import TestClient

from tests.utils import needs_py39, needs_py310


@pytest.fixture(
    name="client",
    params=[
        pytest.param("tutorial002"),
        pytest.param("tutorial002_py39", marks=needs_py39),
        pytest.param("tutorial002_py310", marks=needs_py310),
    ],
)
def get_client(request: pytest.FixtureRequest):
    mod = importlib.import_module(f"docs_src.dataclasses.{request.param}")

    client = TestClient(mod.app)
    client.headers.clear()
    return client


def test_get_item(client: TestClient):
    response = client.get("/items/next")
    assert response.status_code == 200
    assert response.json() == {
        "name": "Island In The Moon",
        "price": 12.99,
        "description": "A place to be playin' and havin' fun",
        "tags": ["breater"],
        "tax": None,
    }


def test_openapi_schema(client: TestClient):
    response = client.get("/openapi.json")
    assert response.status_code == 200
    assert response.json() == {
        "openapi": "3.1.0",
        "info": {"title": "FastAPI", "version": "0.1.0"},
        "paths": {
            "/items/next": {
                "get": {
                    "summary": "Read Next Item",
                    "operationId": "read_next_item_items_next_get",
                    "responses": {
                        "200": {
                            "description": "Successful Response",
                            "content": {
                                "application/json": {
                                    "schema": {"$ref": "#/components/schemas/Item"}
                                }
                            },
                        }
                    },
                }
            }
        },
        "components": {
            "schemas": {
                "Item": {
                    "title": "Item",
                    "required": IsOneOf(
                        ["name", "price", "tags", "description", "tax"],
                        # TODO: remove when deprecating Pydantic v1
                        ["name", "price"],
                    ),
                    "type": "object",
                    "properties": {
                        "name": {"title": "Name", "type": "string"},
                        "price": {"title": "Price", "type": "number"},
                        "tags": IsDict(
                            {
                                "title": "Tags",
                                "type": "array",
                                "items": {"type": "string"},
                            }
                        )
                        | IsDict(
                            # TODO: remove when deprecating Pydantic v1
                            {
                                "title": "Tags",
                                "type": "array",
                                "items": {"type": "string"},
                            }
                        ),
                        "description": IsDict(
                            {
                                "title": "Description",
                                "anyOf": [{"type": "string"}, {"type": "null"}],
                            }
                        )
                        | IsDict(
                            # TODO: remove when deprecating Pydantic v1
                            {"title": "Description", "type": "string"}
                        ),
                        "tax": IsDict(
                            {
                                "title": "Tax",
                                "anyOf": [{"type": "number"}, {"type": "null"}],
                            }
                        )
                        | IsDict(
                            # TODO: remove when deprecating Pydantic v1
                            {"title": "Tax", "type": "number"}
                        ),
                    },
                }
            }
        },
    }



import sys
from typing import Any, List, Union

from tests.utils import pydantic_snapshot, skip_module_if_py_gte_314

if sys.version_info >= (3, 14):
    skip_module_if_py_gte_314()

from fastapi import FastAPI
from fastapi._compat.v1 import BaseModel
from fastapi.testclient import TestClient
from inline_snapshot import snapshot


class SubItem(BaseModel):
    name: str


class Item(BaseModel):
    title: str
    size: int
    description: Union[str, None] = None
    sub: SubItem
    multi: List[SubItem] = []


app = FastAPI()


@app.post("/simple-model")
def handle_simple_model(data: SubItem) -> SubItem:
    return data


@app.post("/simple-model-filter", response_model=SubItem)
def handle_simple_model_filter(data: SubItem) -> Any:
    extended_data = data.dict()
    extended_data.update({"secret_price": 42})
    return extended_data


@app.post("/item")
def handle_item(data: Item) -> Item:
    return data


@app.post("/item-filter", response_model=Item)
def handle_item_filter(data: Item) -> Any:
    extended_data = data.dict()
    extended_data.update({"secret_data": "classified", "internal_id": 12345})
    extended_data["sub"].update({"internal_id": 67890})
    return extended_data


client = TestClient(app)


def test_old_simple_model():
    response = client.post(
        "/simple-model",
        json={"name": "Foo"},
    )
    assert response.status_code == 200, response.text
    assert response.json() == {"name": "Foo"}


def test_old_simple_model_validation_error():
    response = client.post(
        "/simple-model",
        json={"wrong_name": "Foo"},
    )
    assert response.status_code == 422, response.text
    assert response.json() == snapshot(
        {
            "detail": [
                {
                    "loc": ["body", "name"],
                    "msg": "field required",
                    "type": "value_error.missing",
                }
            ]
        }
    )


def test_old_simple_model_filter():
    response = client.post(
        "/simple-model-filter",
        json={"name": "Foo"},
    )
    assert response.status_code == 200, response.text
    assert response.json() == {"name": "Foo"}


def test_item_model():
    response = client.post(
        "/item",
        json={
            "title": "Test Item",
            "size": 100,
            "description": "This is a test item",
            "sub": {"name": "SubItem1"},
            "multi": [{"name": "Multi1"}, {"name": "Multi2"}],
        },
    )
    assert response.status_code == 200, response.text
    assert response.json() == {
        "title": "Test Item",
        "size": 100,
        "description": "This is a test item",
        "sub": {"name": "SubItem1"},
        "multi": [{"name": "Multi1"}, {"name": "Multi2"}],
    }


def test_item_model_minimal():
    response = client.post(
        "/item",
        json={"title": "Minimal Item", "size": 50, "sub": {"name": "SubMin"}},
    )
    assert response.status_code == 200, response.text
    assert response.json() == {
        "title": "Minimal Item",
        "size": 50,
        "description": None,
        "sub": {"name": "SubMin"},
        "multi": [],
    }


def test_item_model_validation_errors():
    response = client.post(
        "/item",
        json={"title": "Missing fields"},
    )
    assert response.status_code == 422, response.text
    error_detail = response.json()["detail"]
    assert len(error_detail) == 2
    assert {
        "loc": ["body", "size"],
        "msg": "field required",
        "type": "value_error.missing",
    } in error_detail
    assert {
        "loc": ["body", "sub"],
        "msg": "field required",
        "type": "value_error.missing",
    } in error_detail


def test_item_model_nested_validation_error():
    response = client.post(
        "/item",
        json={"title": "Test Item", "size": 100, "sub": {"wrong_field": "test"}},
    )
    assert response.status_code == 422, response.text
    assert response.json() == snapshot(
        {
            "detail": [
                {
                    "loc": ["body", "sub", "name"],
                    "msg": "field required",
                    "type": "value_error.missing",
                }
            ]
        }
    )


def test_item_model_invalid_type():
    response = client.post(
        "/item",
        json={"title": "Test Item", "size": "not_a_number", "sub": {"name": "SubItem"}},
    )
    assert response.status_code == 422, response.text
    assert response.json() == snapshot(
        {
            "detail": [
                {
                    "loc": ["body", "size"],
                    "msg": "value is not a valid integer",
                    "type": "type_error.integer",
                }
            ]
        }
    )


def test_item_filter():
    response = client.post(
        "/item-filter",
        json={
            "title": "Filtered Item",
            "size": 200,
            "description": "Test filtering",
            "sub": {"name": "SubFiltered"},
            "multi": [],
        },
    )
    assert response.status_code == 200, response.text
    result = response.json()
    assert result == {
        "title": "Filtered Item",
        "size": 200,
        "description": "Test filtering",
        "sub": {"name": "SubFiltered"},
        "multi": [],
    }
    assert "secret_data" not in result
    assert "internal_id" not in result


def test_openapi_schema():
    response = client.get("/openapi.json")
    assert response.status_code == 200, response.text
    assert response.json() == snapshot(
        {
            "openapi": "3.1.0",
            "info": {"title": "FastAPI", "version": "0.1.0"},
            "paths": {
                "/simple-model": {
                    "post": {
                        "summary": "Handle Simple Model",
                        "operationId": "handle_simple_model_simple_model_post",
                        "requestBody": {
                            "content": {
                                "application/json": {
                                    "schema": pydantic_snapshot(
                                        v2=snapshot(
                                            {
                                                "allOf": [
                                                    {
                                                        "$ref": "#/components/schemas/SubItem"
                                                    }
                                                ],
                                                "title": "Data",
                                            }
                                        ),
                                        v1=snapshot(
                                            {"$ref": "#/components/schemas/SubItem"}
                                        ),
                                    )
                                }
                            },
                            "required": True,
                        },
                        "responses": {
                            "200": {
                                "description": "Successful Response",
                                "content": {
                                    "application/json": {
                                        "schema": {
                                            "$ref": "#/components/schemas/SubItem"
                                        }
                                    }
                                },
                            },
                            "422": {
                                "description": "Validation Error",
                                "content": {
                                    "application/json": {
                                        "schema": {
                                            "$ref": "#/components/schemas/HTTPValidationError"
                                        }
                                    }
                                },
                            },
                        },
                    }
                },
                "/simple-model-filter": {
                    "post": {
                        "summary": "Handle Simple Model Filter",
                        "operationId": "handle_simple_model_filter_simple_model_filter_post",
                        "requestBody": {
                            "content": {
                                "application/json": {
                                    "schema": pydantic_snapshot(
                                        v2=snapshot(
                                            {
                                                "allOf": [
                                                    {
                                                        "$ref": "#/components/schemas/SubItem"
                                                    }
                                                ],
                                                "title": "Data",
                                            }
                                        ),
                                        v1=snapshot(
                                            {"$ref": "#/components/schemas/SubItem"}
                                        ),
                                    )
                                }
                            },
                            "required": True,
                        },
                        "responses": {
                            "200": {
                                "description": "Successful Response",
                                "content": {
                                    "application/json": {
                                        "schema": {
                                            "$ref": "#/components/schemas/SubItem"
                                        }
                                    }
                                },
                            },
                            "422": {
                                "description": "Validation Error",
                                "content": {
                                    "application/json": {
                                        "schema": {
                                            "$ref": "#/components/schemas/HTTPValidationError"
                                        }
                                    }
                                },
                            },
                        },
                    }
                },
                "/item": {
                    "post": {
                        "summary": "Handle Item",
                        "operationId": "handle_item_item_post",
                        "requestBody": {
                            "content": {
                                "application/json": {
                                    "schema": pydantic_snapshot(
                                        v2=snapshot(
                                            {
                                                "allOf": [
                                                    {
                                                        "$ref": "#/components/schemas/Item"
                                                    }
                                                ],
                                                "title": "Data",
                                            }
                                        ),
                                        v1=snapshot(
                                            {"$ref": "#/components/schemas/Item"}
                                        ),
                                    )
                                }
                            },
                            "required": True,
                        },
                        "responses": {
                            "200": {
                                "description": "Successful Response",
                                "content": {
                                    "application/json": {
                                        "schema": {"$ref": "#/components/schemas/Item"}
                                    }
                                },
                            },
                            "422": {
                                "description": "Validation Error",
                                "content": {
                                    "application/json": {
                                        "schema": {
                                            "$ref": "#/components/schemas/HTTPValidationError"
                                        }
                                    }
                                },
                            },
                        },
                    }
                },
                "/item-filter": {
                    "post": {
                        "summary": "Handle Item Filter",
                        "operationId": "handle_item_filter_item_filter_post",
                        "requestBody": {
                            "content": {
                                "application/json": {
                                    "schema": pydantic_snapshot(
                                        v2=snapshot(
                                            {
                                                "allOf": [
                                                    {
                                                        "$ref": "#/components/schemas/Item"
                                                    }
                                                ],
                                                "title": "Data",
                                            }
                                        ),
                                        v1=snapshot(
                                            {"$ref": "#/components/schemas/Item"}
                                        ),
                                    )
                                }
                            },
                            "required": True,
                        },
                        "responses": {
                            "200": {
                                "description": "Successful Response",
                                "content": {
                                    "application/json": {
                                        "schema": {"$ref": "#/components/schemas/Item"}
                                    }
                                },
                            },
                            "422": {
                                "description": "Validation Error",
                                "content": {
                                    "application/json": {
                                        "schema": {
                                            "$ref": "#/components/schemas/HTTPValidationError"
                                        }
                                    }
                                },
                            },
                        },
                    }
                },
            },
            "components": {
                "schemas": {
                    "HTTPValidationError": {
                        "properties": {
                            "detail": {
                                "items": {
                                    "$ref": "#/components/schemas/ValidationError"
                                },
                                "type": "array",
                                "title": "Detail",
                            }
                        },
                        "type": "object",
                        "title": "HTTPValidationError",
                    },
                    "Item": {
                        "properties": {
                            "title": {"type": "string", "title": "Title"},
                            "size": {"type": "integer", "title": "Size"},
                            "description": {"type": "string", "title": "Description"},
                            "sub": {"$ref": "#/components/schemas/SubItem"},
                            "multi": {
                                "items": {"$ref": "#/components/schemas/SubItem"},
                                "type": "array",
                                "title": "Multi",
                                "default": [],
                            },
                        },
                        "type": "object",
                        "required": ["title", "size", "sub"],
                        "title": "Item",
                    },
                    "SubItem": {
                        "properties": {"name": {"type": "string", "title": "Name"}},
                        "type": "object",
                        "required": ["name"],
                        "title": "SubItem",
                    },
                    "ValidationError": {
                        "properties": {
                            "loc": {
                                "items": {
                                    "anyOf": [{"type": "string"}, {"type": "integer"}]
                                },
                                "type": "array",
                                "title": "Location",
                            },
                            "msg": {"type": "string", "title": "Message"},
                            "type": {"type": "string", "title": "Error Type"},
                        },
                        "type": "object",
                        "required": ["loc", "msg", "type"],
                        "title": "ValidationError",
                    },
                }
            },
        }
    )



import importlib
import os
from pathlib import Path

import pytest
from fastapi.testclient import TestClient

from ...utils import needs_py39, needs_py310


@pytest.fixture(
    name="client",
    params=[
        "tutorial002",
        pytest.param("tutorial002_py310", marks=needs_py310),
        "tutorial002_an",
        pytest.param("tutorial002_an_py39", marks=needs_py39),
        pytest.param("tutorial002_an_py310", marks=needs_py310),
    ],
)
def get_client(request: pytest.FixtureRequest):
    mod = importlib.import_module(f"docs_src.background_tasks.{request.param}")

    client = TestClient(mod.app)
    return client


def test(client: TestClient):
    log = Path("log.txt")
    if log.is_file():
        os.remove(log)  # pragma: no cover
    response = client.post("/send-notification/foo@example.com?q=some-query")
    assert response.status_code == 200, response.text
    assert response.json() == {"message": "Message sent"}
    with open("./log.txt") as f:
        assert "found query: some-query\nmessage to foo@example.com" in f.read()



from typing import List

from fastapi import FastAPI
from pydantic import BaseModel

app = FastAPI()


class Item(BaseModel):
    name: str
    price: float


class ResponseMessage(BaseModel):
    message: str


@app.post("/items/", response_model=ResponseMessage)
async def create_item(item: Item):
    return {"message": "item received"}


@app.get("/items/", response_model=List[Item])
async def get_items():
    return [
        {"name": "Plumbus", "price": 3},
        {"name": "Portal Gun", "price": 9001},
    ]



from fastapi.testclient import TestClient

from docs_src.additional_responses.tutorial003 import app

client = TestClient(app)


def test_path_operation():
    response = client.get("/items/foo")
    assert response.status_code == 200, response.text
    assert response.json() == {"id": "foo", "value": "there goes my hero"}


def test_path_operation_not_found():
    response = client.get("/items/bar")
    assert response.status_code == 404, response.text
    assert response.json() == {"message": "Item not found"}


def test_openapi_schema():
    response = client.get("/openapi.json")
    assert response.status_code == 200, response.text
    assert response.json() == {
        "openapi": "3.1.0",
        "info": {"title": "FastAPI", "version": "0.1.0"},
        "paths": {
            "/items/{item_id}": {
                "get": {
                    "responses": {
                        "404": {
                            "description": "The item was not found",
                            "content": {
                                "application/json": {
                                    "schema": {"$ref": "#/components/schemas/Message"}
                                }
                            },
                        },
                        "200": {
                            "description": "Item requested by ID",
                            "content": {
                                "application/json": {
                                    "schema": {"$ref": "#/components/schemas/Item"},
                                    "example": {
                                        "id": "bar",
                                        "value": "The bar tenders",
                                    },
                                }
                            },
                        },
                        "422": {
                            "description": "Validation Error",
                            "content": {
                                "application/json": {
                                    "schema": {
                                        "$ref": "#/components/schemas/HTTPValidationError"
                                    }
                                }
                            },
                        },
                    },
                    "summary": "Read Item",
                    "operationId": "read_item_items__item_id__get",
                    "parameters": [
                        {
                            "required": True,
                            "schema": {"title": "Item Id", "type": "string"},
                            "name": "item_id",
                            "in": "path",
                        }
                    ],
                }
            }
        },
        "components": {
            "schemas": {
                "Item": {
                    "title": "Item",
                    "required": ["id", "value"],
                    "type": "object",
                    "properties": {
                        "id": {"title": "Id", "type": "string"},
                        "value": {"title": "Value", "type": "string"},
                    },
                },
                "Message": {
                    "title": "Message",
                    "required": ["message"],
                    "type": "object",
                    "properties": {"message": {"title": "Message", "type": "string"}},
                },
                "ValidationError": {
                    "title": "ValidationError",
                    "required": ["loc", "msg", "type"],
                    "type": "object",
                    "properties": {
                        "loc": {
                            "title": "Location",
                            "type": "array",
                            "items": {
                                "anyOf": [{"type": "string"}, {"type": "integer"}]
                            },
                        },
                        "msg": {"title": "Message", "type": "string"},
                        "type": {"title": "Error Type", "type": "string"},
                    },
                },
                "HTTPValidationError": {
                    "title": "HTTPValidationError",
                    "type": "object",
                    "properties": {
                        "detail": {
                            "title": "Detail",
                            "type": "array",
                            "items": {"$ref": "#/components/schemas/ValidationError"},
                        }
                    },
                },
            }
        },
    }



from typing import Annotated, Union

from fastapi import Body, FastAPI
from pydantic import BaseModel

app = FastAPI()


class Item(BaseModel):
    name: str
    description: Union[str, None] = None
    price: float
    tax: Union[float, None] = None


@app.put("/items/{item_id}")
async def update_item(
    *,
    item_id: int,
    item: Annotated[
        Item,
        Body(
            examples=[
                {
                    "name": "Foo",
                    "description": "A very nice Item",
                    "price": 35.4,
                    "tax": 3.2,
                },
                {
                    "name": "Bar",
                    "price": "35.4",
                },
                {
                    "name": "Baz",
                    "price": "thirty five point four",
                },
            ],
        ),
    ],
):
    results = {"item_id": item_id, "item": item}
    return results



from fastapi import FastAPI, Query
from typing_extensions import Annotated

app = FastAPI()


@app.get("/items/")
async def read_items(q: Annotated[list, Query()] = []):
    query_items = {"q": q}
    return query_items



from typing import List, Optional

from fastapi import FastAPI, File
from fastapi.testclient import TestClient

app = FastAPI()


@app.post("/files")
async def upload_files(files: Optional[List[bytes]] = File(None)):
    if files is None:
        return {"files_count": 0}
    return {"files_count": len(files), "sizes": [len(f) for f in files]}


def test_optional_bytes_list():
    client = TestClient(app)
    response = client.post(
        "/files",
        files=[("files", b"content1"), ("files", b"content2")],
    )
    assert response.status_code == 200
    assert response.json() == {"files_count": 2, "sizes": [8, 8]}


def test_optional_bytes_list_no_files():
    client = TestClient(app)
    response = client.post("/files")
    assert response.status_code == 200
    assert response.json() == {"files_count": 0}



import sys
from typing import Any

import pytest
from fastapi._compat import PYDANTIC_V2

from tests.utils import skip_module_if_py_gte_314

if sys.version_info >= (3, 14):
    skip_module_if_py_gte_314()


if not PYDANTIC_V2:
    pytest.skip("This test is only for Pydantic v2", allow_module_level=True)

import importlib

import pytest

from ...utils import needs_py310


@pytest.fixture(
    name="mod",
    params=[
        "tutorial001_an",
        pytest.param("tutorial001_an_py310", marks=needs_py310),
    ],
)
def get_mod(request: pytest.FixtureRequest):
    mod = importlib.import_module(f"docs_src.pydantic_v1_in_v2.{request.param}")
    return mod


def test_model(mod: Any):
    item = mod.Item(name="Foo", size=3.4)
    assert item.dict() == {"name": "Foo", "description": None, "size": 3.4}



from dataclasses import field  # (1)

from fastapi import FastAPI
from pydantic.dataclasses import dataclass  # (2)


@dataclass
class Item:
    name: str
    description: str | None = None


@dataclass
class Author:
    name: str
    items: list[Item] = field(default_factory=list)  # (3)


app = FastAPI()


@app.post("/authors/{author_id}/items/", response_model=Author)  # (4)
async def create_author_items(author_id: str, items: list[Item]):  # (5)
    return {"name": author_id, "items": items}  # (6)


@app.get("/authors/", response_model=list[Author])  # (7)
def get_authors():  # (8)
    return [  # (9)
        {
            "name": "Breaters",
            "items": [
                {
                    "name": "Island In The Moon",
                    "description": "A place to be playin' and havin' fun",
                },
                {"name": "Holy Buddies"},
            ],
        },
        {
            "name": "System of an Up",
            "items": [
                {
                    "name": "Salt",
                    "description": "The kombucha mushroom people's favorite",
                },
                {"name": "Pad Thai"},
                {
                    "name": "Lonely Night",
                    "description": "The mostests lonliest nightiest of allest",
                },
            ],
        },
    ]



from typing import List, Tuple

from dirty_equals import IsDict
from fastapi import FastAPI, Form
from fastapi.testclient import TestClient
from pydantic import BaseModel

app = FastAPI()


class ItemGroup(BaseModel):
    items: List[Tuple[str, str]]


class Coordinate(BaseModel):
    x: float
    y: float


@app.post("/model-with-tuple/")
def post_model_with_tuple(item_group: ItemGroup):
    return item_group


@app.post("/tuple-of-models/")
def post_tuple_of_models(square: Tuple[Coordinate, Coordinate]):
    return square


@app.post("/tuple-form/")
def hello(values: Tuple[int, int] = Form()):
    return values


client = TestClient(app)


def test_model_with_tuple_valid():
    data = {"items": [["foo", "bar"], ["baz", "whatelse"]]}
    response = client.post("/model-with-tuple/", json=data)
    assert response.status_code == 200, response.text
    assert response.json() == data


def test_model_with_tuple_invalid():
    data = {"items": [["foo", "bar"], ["baz", "whatelse", "too", "much"]]}
    response = client.post("/model-with-tuple/", json=data)
    assert response.status_code == 422, response.text

    data = {"items": [["foo", "bar"], ["baz"]]}
    response = client.post("/model-with-tuple/", json=data)
    assert response.status_code == 422, response.text


def test_tuple_with_model_valid():
    data = [{"x": 1, "y": 2}, {"x": 3, "y": 4}]
    response = client.post("/tuple-of-models/", json=data)
    assert response.status_code == 200, response.text
    assert response.json() == data


def test_tuple_with_model_invalid():
    data = [{"x": 1, "y": 2}, {"x": 3, "y": 4}, {"x": 5, "y": 6}]
    response = client.post("/tuple-of-models/", json=data)
    assert response.status_code == 422, response.text

    data = [{"x": 1, "y": 2}]
    response = client.post("/tuple-of-models/", json=data)
    assert response.status_code == 422, response.text


def test_tuple_form_valid():
    response = client.post("/tuple-form/", data={"values": ("1", "2")})
    assert response.status_code == 200, response.text
    assert response.json() == [1, 2]


def test_tuple_form_invalid():
    response = client.post("/tuple-form/", data={"values": ("1", "2", "3")})
    assert response.status_code == 422, response.text

    response = client.post("/tuple-form/", data={"values": ("1")})
    assert response.status_code == 422, response.text


def test_openapi_schema():
    response = client.get("/openapi.json")
    assert response.status_code == 200, response.text
    assert response.json() == {
        "openapi": "3.1.0",
        "info": {"title": "FastAPI", "version": "0.1.0"},
        "paths": {
            "/model-with-tuple/": {
                "post": {
                    "summary": "Post Model With Tuple",
                    "operationId": "post_model_with_tuple_model_with_tuple__post",
                    "requestBody": {
                        "content": {
                            "application/json": {
                                "schema": {"$ref": "#/components/schemas/ItemGroup"}
                            }
                        },
                        "required": True,
                    },
                    "responses": {
                        "200": {
                            "description": "Successful Response",
                            "content": {"application/json": {"schema": {}}},
                        },
                        "422": {
                            "description": "Validation Error",
                            "content": {
                                "application/json": {
                                    "schema": {
                                        "$ref": "#/components/schemas/HTTPValidationError"
                                    }
                                }
                            },
                        },
                    },
                }
            },
            "/tuple-of-models/": {
                "post": {
                    "summary": "Post Tuple Of Models",
                    "operationId": "post_tuple_of_models_tuple_of_models__post",
                    "requestBody": {
                        "content": {
                            "application/json": {
                                "schema": IsDict(
                                    {
                                        "title": "Square",
                                        "maxItems": 2,
                                        "minItems": 2,
                                        "type": "array",
                                        "prefixItems": [
                                            {"$ref": "#/components/schemas/Coordinate"},
                                            {"$ref": "#/components/schemas/Coordinate"},
                                        ],
                                    }
                                )
                                | IsDict(
                                    # TODO: remove when deprecating Pydantic v1
                                    {
                                        "title": "Square",
                                        "maxItems": 2,
                                        "minItems": 2,
                                        "type": "array",
                                        "items": [
                                            {"$ref": "#/components/schemas/Coordinate"},
                                            {"$ref": "#/components/schemas/Coordinate"},
                                        ],
                                    }
                                )
                            }
                        },
                        "required": True,
                    },
                    "responses": {
                        "200": {
                            "description": "Successful Response",
                            "content": {"application/json": {"schema": {}}},
                        },
                        "422": {
                            "description": "Validation Error",
                            "content": {
                                "application/json": {
                                    "schema": {
                                        "$ref": "#/components/schemas/HTTPValidationError"
                                    }
                                }
                            },
                        },
                    },
                }
            },
            "/tuple-form/": {
                "post": {
                    "summary": "Hello",
                    "operationId": "hello_tuple_form__post",
                    "requestBody": {
                        "content": {
                            "application/x-www-form-urlencoded": {
                                "schema": {
                                    "$ref": "#/components/schemas/Body_hello_tuple_form__post"
                                }
                            }
                        },
                        "required": True,
                    },
                    "responses": {
                        "200": {
                            "description": "Successful Response",
                            "content": {"application/json": {"schema": {}}},
                        },
                        "422": {
                            "description": "Validation Error",
                            "content": {
                                "application/json": {
                                    "schema": {
                                        "$ref": "#/components/schemas/HTTPValidationError"
                                    }
                                }
                            },
                        },
                    },
                }
            },
        },
        "components": {
            "schemas": {
                "Body_hello_tuple_form__post": {
                    "title": "Body_hello_tuple_form__post",
                    "required": ["values"],
                    "type": "object",
                    "properties": {
                        "values": IsDict(
                            {
                                "title": "Values",
                                "maxItems": 2,
                                "minItems": 2,
                                "type": "array",
                                "prefixItems": [
                                    {"type": "integer"},
                                    {"type": "integer"},
                                ],
                            }
                        )
                        | IsDict(
                            # TODO: remove when deprecating Pydantic v1
                            {
                                "title": "Values",
                                "maxItems": 2,
                                "minItems": 2,
                                "type": "array",
                                "items": [{"type": "integer"}, {"type": "integer"}],
                            }
                        )
                    },
                },
                "Coordinate": {
                    "title": "Coordinate",
                    "required": ["x", "y"],
                    "type": "object",
                    "properties": {
                        "x": {"title": "X", "type": "number"},
                        "y": {"title": "Y", "type": "number"},
                    },
                },
                "HTTPValidationError": {
                    "title": "HTTPValidationError",
                    "type": "object",
                    "properties": {
                        "detail": {
                            "title": "Detail",
                            "type": "array",
                            "items": {"$ref": "#/components/schemas/ValidationError"},
                        }
                    },
                },
                "ItemGroup": {
                    "title": "ItemGroup",
                    "required": ["items"],
                    "type": "object",
                    "properties": {
                        "items": {
                            "title": "Items",
                            "type": "array",
                            "items": IsDict(
                                {
                                    "maxItems": 2,
                                    "minItems": 2,
                                    "type": "array",
                                    "prefixItems": [
                                        {"type": "string"},
                                        {"type": "string"},
                                    ],
                                }
                            )
                            | IsDict(
                                # TODO: remove when deprecating Pydantic v1
                                {
                                    "maxItems": 2,
                                    "minItems": 2,
                                    "type": "array",
                                    "items": [{"type": "string"}, {"type": "string"}],
                                }
                            ),
                        }
                    },
                },
                "ValidationError": {
                    "title": "ValidationError",
                    "required": ["loc", "msg", "type"],
                    "type": "object",
                    "properties": {
                        "loc": {
                            "title": "Location",
                            "type": "array",
                            "items": {
                                "anyOf": [{"type": "string"}, {"type": "integer"}]
                            },
                        },
                        "msg": {"title": "Message", "type": "string"},
                        "type": {"title": "Error Type", "type": "string"},
                    },
                },
            }
        },
    }



from fastapi import FastAPI
from fastapi.testclient import TestClient

app = FastAPI()


@app.get("/")
async def read_main():
    return {"msg": "Hello World"}


client = TestClient(app)


def test_read_main():
    response = client.get("/")
    assert response.status_code == 200
    assert response.json() == {"msg": "Hello World"}



import gzip
from typing import Callable, List

from fastapi import Body, FastAPI, Request, Response
from fastapi.routing import APIRoute
from typing_extensions import Annotated


class GzipRequest(Request):
    async def body(self) -> bytes:
        if not hasattr(self, "_body"):
            body = await super().body()
            if "gzip" in self.headers.getlist("Content-Encoding"):
                body = gzip.decompress(body)
            self._body = body
        return self._body


class GzipRoute(APIRoute):
    def get_route_handler(self) -> Callable:
        original_route_handler = super().get_route_handler()

        async def custom_route_handler(request: Request) -> Response:
            request = GzipRequest(request.scope, request.receive)
            return await original_route_handler(request)

        return custom_route_handler


app = FastAPI()
app.router.route_class = GzipRoute


@app.post("/sum")
async def sum_numbers(numbers: Annotated[List[int], Body()]):
    return {"sum": sum(numbers)}



from fastapi import FastAPI, HTTPException
from fastapi.testclient import TestClient
from starlette.exceptions import HTTPException as StarletteHTTPException

app = FastAPI()

items = {"foo": "The Foo Wrestlers"}


@app.get("/items/{item_id}")
async def read_item(item_id: str):
    if item_id not in items:
        raise HTTPException(
            status_code=404,
            detail="Item not found",
            headers={"X-Error": "Some custom header"},
        )
    return {"item": items[item_id]}


@app.get("/http-no-body-statuscode-exception")
async def no_body_status_code_exception():
    raise HTTPException(status_code=204)


@app.get("/http-no-body-statuscode-with-detail-exception")
async def no_body_status_code_with_detail_exception():
    raise HTTPException(status_code=204, detail="I should just disappear!")


@app.get("/starlette-items/{item_id}")
async def read_starlette_item(item_id: str):
    if item_id not in items:
        raise StarletteHTTPException(status_code=404, detail="Item not found")
    return {"item": items[item_id]}


client = TestClient(app)


def test_get_item():
    response = client.get("/items/foo")
    assert response.status_code == 200, response.text
    assert response.json() == {"item": "The Foo Wrestlers"}


def test_get_item_not_found():
    response = client.get("/items/bar")
    assert response.status_code == 404, response.text
    assert response.headers.get("x-error") == "Some custom header"
    assert response.json() == {"detail": "Item not found"}


def test_get_starlette_item():
    response = client.get("/starlette-items/foo")
    assert response.status_code == 200, response.text
    assert response.json() == {"item": "The Foo Wrestlers"}


def test_get_starlette_item_not_found():
    response = client.get("/starlette-items/bar")
    assert response.status_code == 404, response.text
    assert response.headers.get("x-error") is None
    assert response.json() == {"detail": "Item not found"}


def test_no_body_status_code_exception_handlers():
    response = client.get("/http-no-body-statuscode-exception")
    assert response.status_code == 204
    assert not response.content


def test_no_body_status_code_with_detail_exception_handlers():
    response = client.get("/http-no-body-statuscode-with-detail-exception")
    assert response.status_code == 204
    assert not response.content


def test_openapi_schema():
    response = client.get("/openapi.json")
    assert response.status_code == 200, response.text
    assert response.json() == {
        "openapi": "3.1.0",
        "info": {"title": "FastAPI", "version": "0.1.0"},
        "paths": {
            "/http-no-body-statuscode-exception": {
                "get": {
                    "operationId": "no_body_status_code_exception_http_no_body_statuscode_exception_get",
                    "responses": {
                        "200": {
                            "content": {"application/json": {"schema": {}}},
                            "description": "Successful Response",
                        }
                    },
                    "summary": "No Body Status Code Exception",
                }
            },
            "/http-no-body-statuscode-with-detail-exception": {
                "get": {
                    "operationId": "no_body_status_code_with_detail_exception_http_no_body_statuscode_with_detail_exception_get",
                    "responses": {
                        "200": {
                            "content": {"application/json": {"schema": {}}},
                            "description": "Successful Response",
                        }
                    },
                    "summary": "No Body Status Code With Detail Exception",
                }
            },
            "/items/{item_id}": {
                "get": {
                    "responses": {
                        "200": {
                            "description": "Successful Response",
                            "content": {"application/json": {"schema": {}}},
                        },
                        "422": {
                            "description": "Validation Error",
                            "content": {
                                "application/json": {
                                    "schema": {
                                        "$ref": "#/components/schemas/HTTPValidationError"
                                    }
                                }
                            },
                        },
                    },
                    "summary": "Read Item",
                    "operationId": "read_item_items__item_id__get",
                    "parameters": [
                        {
                            "required": True,
                            "schema": {"title": "Item Id", "type": "string"},
                            "name": "item_id",
                            "in": "path",
                        }
                    ],
                }
            },
            "/starlette-items/{item_id}": {
                "get": {
                    "responses": {
                        "200": {
                            "description": "Successful Response",
                            "content": {"application/json": {"schema": {}}},
                        },
                        "422": {
                            "description": "Validation Error",
                            "content": {
                                "application/json": {
                                    "schema": {
                                        "$ref": "#/components/schemas/HTTPValidationError"
                                    }
                                }
                            },
                        },
                    },
                    "summary": "Read Starlette Item",
                    "operationId": "read_starlette_item_starlette_items__item_id__get",
                    "parameters": [
                        {
                            "required": True,
                            "schema": {"title": "Item Id", "type": "string"},
                            "name": "item_id",
                            "in": "path",
                        }
                    ],
                }
            },
        },
        "components": {
            "schemas": {
                "ValidationError": {
                    "title": "ValidationError",
                    "required": ["loc", "msg", "type"],
                    "type": "object",
                    "properties": {
                        "loc": {
                            "title": "Location",
                            "type": "array",
                            "items": {
                                "anyOf": [{"type": "string"}, {"type": "integer"}]
                            },
                        },
                        "msg": {"title": "Message", "type": "string"},
                        "type": {"title": "Error Type", "type": "string"},
                    },
                },
                "HTTPValidationError": {
                    "title": "HTTPValidationError",
                    "type": "object",
                    "properties": {
                        "detail": {
                            "title": "Detail",
                            "type": "array",
                            "items": {"$ref": "#/components/schemas/ValidationError"},
                        }
                    },
                },
            }
        },
    }



import importlib

import pytest
from fastapi.testclient import TestClient
from pytest import MonkeyPatch

from ...utils import needs_pydanticv1, needs_pydanticv2


@pytest.fixture(
    name="app",
    params=[
        pytest.param("tutorial001", marks=needs_pydanticv2),
        pytest.param("tutorial001_pv1", marks=needs_pydanticv1),
    ],
)
def get_app(request: pytest.FixtureRequest, monkeypatch: MonkeyPatch):
    monkeypatch.setenv("ADMIN_EMAIL", "admin@example.com")
    mod = importlib.import_module(f"docs_src.settings.{request.param}")
    return mod.app


def test_settings(app):
    client = TestClient(app)
    response = client.get("/info")
    assert response.status_code == 200, response.text
    assert response.json() == {
        "app_name": "Awesome API",
        "admin_email": "admin@example.com",
        "items_per_user": 50,
    }



from fastapi import FastAPI
from pydantic import BaseModel, Field

app = FastAPI()


class Item(BaseModel):
    name: str = Field(examples=["Foo"])
    description: str | None = Field(default=None, examples=["A very nice Item"])
    price: float = Field(examples=[35.4])
    tax: float | None = Field(default=None, examples=[3.2])


@app.put("/items/{item_id}")
async def update_item(item_id: int, item: Item):
    results = {"item_id": item_id, "item": item}
    return results



from typing import Annotated, Union

from fastapi import (
    Cookie,
    Depends,
    FastAPI,
    Query,
    WebSocket,
    WebSocketException,
    status,
)
from fastapi.responses import HTMLResponse

app = FastAPI()

html = """
<!DOCTYPE html>
<html>
    <head>
        <title>Chat</title>
    </head>
    <body>
        <h1>WebSocket Chat</h1>
        <form action="" onsubmit="sendMessage(event)">
            <label>Item ID: <input type="text" id="itemId" autocomplete="off" value="foo"/></label>
            <label>Token: <input type="text" id="token" autocomplete="off" value="some-key-token"/></label>
            <button onclick="connect(event)">Connect</button>
            <hr>
            <label>Message: <input type="text" id="messageText" autocomplete="off"/></label>
            <button>Send</button>
        </form>
        <ul id='messages'>
        </ul>
        <script>
        var ws = null;
            function connect(event) {
                var itemId = document.getElementById("itemId")
                var token = document.getElementById("token")
                ws = new WebSocket("ws://localhost:8000/items/" + itemId.value + "/ws?token=" + token.value);
                ws.onmessage = function(event) {
                    var messages = document.getElementById('messages')
                    var message = document.createElement('li')
                    var content = document.createTextNode(event.data)
                    message.appendChild(content)
                    messages.appendChild(message)
                };
                event.preventDefault()
            }
            function sendMessage(event) {
                var input = document.getElementById("messageText")
                ws.send(input.value)
                input.value = ''
                event.preventDefault()
            }
        </script>
    </body>
</html>
"""


@app.get("/")
async def get():
    return HTMLResponse(html)


async def get_cookie_or_token(
    websocket: WebSocket,
    session: Annotated[Union[str, None], Cookie()] = None,
    token: Annotated[Union[str, None], Query()] = None,
):
    if session is None and token is None:
        raise WebSocketException(code=status.WS_1008_POLICY_VIOLATION)
    return session or token


@app.websocket("/items/{item_id}/ws")
async def websocket_endpoint(
    *,
    websocket: WebSocket,
    item_id: str,
    q: Union[int, None] = None,
    cookie_or_token: Annotated[str, Depends(get_cookie_or_token)],
):
    await websocket.accept()
    while True:
        data = await websocket.receive_text()
        await websocket.send_text(
            f"Session cookie or query token value is: {cookie_or_token}"
        )
        if q is not None:
            await websocket.send_text(f"Query parameter q is: {q}")
        await websocket.send_text(f"Message text was: {data}, for item ID: {item_id}")



import sys

from tests.utils import pydantic_snapshot, skip_module_if_py_gte_314

if sys.version_info >= (3, 14):
    skip_module_if_py_gte_314()

from fastapi.testclient import TestClient
from inline_snapshot import snapshot

from .main import app

client = TestClient(app)


def test_v1_to_v2_item():
    response = client.post(
        "/v1-to-v2/item",
        json={"title": "Test", "size": 10, "sub": {"name": "SubTest"}},
    )
    assert response.status_code == 200
    assert response.json() == {
        "new_title": "Test",
        "new_size": 10,
        "new_description": None,
        "new_sub": {"new_sub_name": "SubTest"},
        "new_multi": [],
    }


def test_v2_to_v1_item():
    response = client.post(
        "/v2-to-v1/item",
        json={
            "new_title": "NewTest",
            "new_size": 20,
            "new_sub": {"new_sub_name": "NewSubTest"},
        },
    )
    assert response.status_code == 200
    assert response.json() == {
        "title": "NewTest",
        "size": 20,
        "description": None,
        "sub": {"name": "NewSubTest"},
        "multi": [],
    }


def test_v1_to_v2_item_to_list():
    response = client.post(
        "/v1-to-v2/item-to-list",
        json={"title": "ListTest", "size": 30, "sub": {"name": "SubListTest"}},
    )
    assert response.status_code == 200
    assert response.json() == [
        {
            "new_title": "ListTest",
            "new_size": 30,
            "new_description": None,
            "new_sub": {"new_sub_name": "SubListTest"},
            "new_multi": [],
        },
        {
            "new_title": "ListTest",
            "new_size": 30,
            "new_description": None,
            "new_sub": {"new_sub_name": "SubListTest"},
            "new_multi": [],
        },
    ]


def test_v1_to_v2_list_to_list():
    response = client.post(
        "/v1-to-v2/list-to-list",
        json=[
            {"title": "Item1", "size": 40, "sub": {"name": "Sub1"}},
            {"title": "Item2", "size": 50, "sub": {"name": "Sub2"}},
        ],
    )
    assert response.status_code == 200
    assert response.json() == [
        {
            "new_title": "Item1",
            "new_size": 40,
            "new_description": None,
            "new_sub": {"new_sub_name": "Sub1"},
            "new_multi": [],
        },
        {
            "new_title": "Item2",
            "new_size": 50,
            "new_description": None,
            "new_sub": {"new_sub_name": "Sub2"},
            "new_multi": [],
        },
    ]


def test_v1_to_v2_list_to_item():
    response = client.post(
        "/v1-to-v2/list-to-item",
        json=[
            {"title": "FirstItem", "size": 60, "sub": {"name": "FirstSub"}},
            {"title": "SecondItem", "size": 70, "sub": {"name": "SecondSub"}},
        ],
    )
    assert response.status_code == 200
    assert response.json() == {
        "new_title": "FirstItem",
        "new_size": 60,
        "new_description": None,
        "new_sub": {"new_sub_name": "FirstSub"},
        "new_multi": [],
    }


def test_v2_to_v1_item_to_list():
    response = client.post(
        "/v2-to-v1/item-to-list",
        json={
            "new_title": "ListNew",
            "new_size": 80,
            "new_sub": {"new_sub_name": "SubListNew"},
        },
    )
    assert response.status_code == 200
    assert response.json() == [
        {
            "title": "ListNew",
            "size": 80,
            "description": None,
            "sub": {"name": "SubListNew"},
            "multi": [],
        },
        {
            "title": "ListNew",
            "size": 80,
            "description": None,
            "sub": {"name": "SubListNew"},
            "multi": [],
        },
    ]


def test_v2_to_v1_list_to_list():
    response = client.post(
        "/v2-to-v1/list-to-list",
        json=[
            {
                "new_title": "New1",
                "new_size": 90,
                "new_sub": {"new_sub_name": "NewSub1"},
            },
            {
                "new_title": "New2",
                "new_size": 100,
                "new_sub": {"new_sub_name": "NewSub2"},
            },
        ],
    )
    assert response.status_code == 200
    assert response.json() == [
        {
            "title": "New1",
            "size": 90,
            "description": None,
            "sub": {"name": "NewSub1"},
            "multi": [],
        },
        {
            "title": "New2",
            "size": 100,
            "description": None,
            "sub": {"name": "NewSub2"},
            "multi": [],
        },
    ]


def test_v2_to_v1_list_to_item():
    response = client.post(
        "/v2-to-v1/list-to-item",
        json=[
            {
                "new_title": "FirstNew",
                "new_size": 110,
                "new_sub": {"new_sub_name": "FirstNewSub"},
            },
            {
                "new_title": "SecondNew",
                "new_size": 120,
                "new_sub": {"new_sub_name": "SecondNewSub"},
            },
        ],
    )
    assert response.status_code == 200
    assert response.json() == {
        "title": "FirstNew",
        "size": 110,
        "description": None,
        "sub": {"name": "FirstNewSub"},
        "multi": [],
    }


def test_v1_to_v2_list_to_item_empty():
    response = client.post("/v1-to-v2/list-to-item", json=[])
    assert response.status_code == 200
    assert response.json() == {
        "new_title": "",
        "new_size": 0,
        "new_description": None,
        "new_sub": {"new_sub_name": ""},
        "new_multi": [],
    }


def test_v2_to_v1_list_to_item_empty():
    response = client.post("/v2-to-v1/list-to-item", json=[])
    assert response.status_code == 200
    assert response.json() == {
        "title": "",
        "size": 0,
        "description": None,
        "sub": {"name": ""},
        "multi": [],
    }


def test_v2_same_name_to_v1():
    response = client.post(
        "/v2-to-v1/same-name",
        json={
            "item1": {
                "new_title": "Title1",
                "new_size": 100,
                "new_description": "Description1",
                "new_sub": {"new_sub_name": "Sub1"},
                "new_multi": [{"new_sub_name": "Multi1"}],
            },
            "item2": {
                "dup_title": "Title2",
                "dup_size": 200,
                "dup_description": "Description2",
                "dup_sub": {"dup_sub_name": "Sub2"},
                "dup_multi": [
                    {"dup_sub_name": "Multi2a"},
                    {"dup_sub_name": "Multi2b"},
                ],
            },
        },
    )
    assert response.status_code == 200
    assert response.json() == {
        "title": "Title1",
        "size": 200,
        "description": "Description1",
        "sub": {"name": "Sub1"},
        "multi": [{"name": "Multi2a"}, {"name": "Multi2b"}],
    }


def test_v2_items_in_list_to_v1_item_in_list():
    response = client.post(
        "/v2-to-v1/list-of-items-to-list-of-items",
        json={
            "data1": [{"name2": "Item1"}, {"name2": "Item2"}],
            "data2": [{"dup_name2": "Item3"}, {"dup_name2": "Item4"}],
        },
    )
    assert response.status_code == 200, response.text
    assert response.json() == [
        {"name1": "Item1"},
        {"name1": "Item3"},
    ]


def test_openapi_schema():
    response = client.get("/openapi.json")
    assert response.status_code == 200
    assert response.json() == snapshot(
        {
            "openapi": "3.1.0",
            "info": {"title": "FastAPI", "version": "0.1.0"},
            "paths": {
                "/v1-to-v2/item": {
                    "post": {
                        "summary": "Handle V1 Item To V2",
                        "operationId": "handle_v1_item_to_v2_v1_to_v2_item_post",
                        "requestBody": {
                            "content": {
                                "application/json": {
                                    "schema": pydantic_snapshot(
                                        v2=snapshot(
                                            {
                                                "allOf": [
                                                    {
                                                        "$ref": "#/components/schemas/tests__test_pydantic_v1_v2_multifile__modelsv1__Item"
                                                    }
                                                ],
                                                "title": "Data",
                                            }
                                        ),
                                        v1=snapshot(
                                            {
                                                "$ref": "#/components/schemas/tests__test_pydantic_v1_v2_multifile__modelsv1__Item"
                                            }
                                        ),
                                    )
                                }
                            },
                            "required": True,
                        },
                        "responses": {
                            "200": {
                                "description": "Successful Response",
                                "content": {
                                    "application/json": {
                                        "schema": {
                                            "$ref": "#/components/schemas/tests__test_pydantic_v1_v2_multifile__modelsv2__Item"
                                        }
                                    }
                                },
                            },
                            "422": {
                                "description": "Validation Error",
                                "content": {
                                    "application/json": {
                                        "schema": {
                                            "$ref": "#/components/schemas/HTTPValidationError"
                                        }
                                    }
                                },
                            },
                        },
                    }
                },
                "/v2-to-v1/item": {
                    "post": {
                        "summary": "Handle V2 Item To V1",
                        "operationId": "handle_v2_item_to_v1_v2_to_v1_item_post",
                        "requestBody": {
                            "content": {
                                "application/json": {
                                    "schema": pydantic_snapshot(
                                        v2=snapshot(
                                            {
                                                "$ref": "#/components/schemas/tests__test_pydantic_v1_v2_multifile__modelsv2__Item-Input"
                                            }
                                        ),
                                        v1=snapshot(
                                            {
                                                "$ref": "#/components/schemas/tests__test_pydantic_v1_v2_multifile__modelsv2__Item"
                                            }
                                        ),
                                    ),
                                }
                            },
                            "required": True,
                        },
                        "responses": {
                            "200": {
                                "description": "Successful Response",
                                "content": {
                                    "application/json": {
                                        "schema": {
                                            "$ref": "#/components/schemas/tests__test_pydantic_v1_v2_multifile__modelsv1__Item"
                                        }
                                    }
                                },
                            },
                            "422": {
                                "description": "Validation Error",
                                "content": {
                                    "application/json": {
                                        "schema": {
                                            "$ref": "#/components/schemas/HTTPValidationError"
                                        }
                                    }
                                },
                            },
                        },
                    }
                },
                "/v1-to-v2/item-to-list": {
                    "post": {
                        "summary": "Handle V1 Item To V2 List",
                        "operationId": "handle_v1_item_to_v2_list_v1_to_v2_item_to_list_post",
                        "requestBody": {
                            "content": {
                                "application/json": {
                                    "schema": pydantic_snapshot(
                                        v2=snapshot(
                                            {
                                                "allOf": [
                                                    {
                                                        "$ref": "#/components/schemas/tests__test_pydantic_v1_v2_multifile__modelsv1__Item"
                                                    }
                                                ],
                                                "title": "Data",
                                            }
                                        ),
                                        v1=snapshot(
                                            {
                                                "$ref": "#/components/schemas/tests__test_pydantic_v1_v2_multifile__modelsv1__Item"
                                            }
                                        ),
                                    )
                                }
                            },
                            "required": True,
                        },
                        "responses": {
                            "200": {
                                "description": "Successful Response",
                                "content": {
                                    "application/json": {
                                        "schema": {
                                            "items": {
                                                "$ref": "#/components/schemas/tests__test_pydantic_v1_v2_multifile__modelsv2__Item"
                                            },
                                            "type": "array",
                                            "title": "Response Handle V1 Item To V2 List V1 To V2 Item To List Post",
                                        }
                                    }
                                },
                            },
                            "422": {
                                "description": "Validation Error",
                                "content": {
                                    "application/json": {
                                        "schema": {
                                            "$ref": "#/components/schemas/HTTPValidationError"
                                        }
                                    }
                                },
                            },
                        },
                    }
                },
                "/v1-to-v2/list-to-list": {
                    "post": {
                        "summary": "Handle V1 List To V2 List",
                        "operationId": "handle_v1_list_to_v2_list_v1_to_v2_list_to_list_post",
                        "requestBody": {
                            "content": {
                                "application/json": {
                                    "schema": {
                                        "items": {
                                            "$ref": "#/components/schemas/tests__test_pydantic_v1_v2_multifile__modelsv1__Item"
                                        },
                                        "type": "array",
                                        "title": "Data",
                                    }
                                }
                            },
                            "required": True,
                        },
                        "responses": {
                            "200": {
                                "description": "Successful Response",
                                "content": {
                                    "application/json": {
                                        "schema": {
                                            "items": {
                                                "$ref": "#/components/schemas/tests__test_pydantic_v1_v2_multifile__modelsv2__Item"
                                            },
                                            "type": "array",
                                            "title": "Response Handle V1 List To V2 List V1 To V2 List To List Post",
                                        }
                                    }
                                },
                            },
                            "422": {
                                "description": "Validation Error",
                                "content": {
                                    "application/json": {
                                        "schema": {
                                            "$ref": "#/components/schemas/HTTPValidationError"
                                        }
                                    }
                                },
                            },
                        },
                    }
                },
                "/v1-to-v2/list-to-item": {
                    "post": {
                        "summary": "Handle V1 List To V2 Item",
                        "operationId": "handle_v1_list_to_v2_item_v1_to_v2_list_to_item_post",
                        "requestBody": {
                            "content": {
                                "application/json": {
                                    "schema": {
                                        "items": {
                                            "$ref": "#/components/schemas/tests__test_pydantic_v1_v2_multifile__modelsv1__Item"
                                        },
                                        "type": "array",
                                        "title": "Data",
                                    }
                                }
                            },
                            "required": True,
                        },
                        "responses": {
                            "200": {
                                "description": "Successful Response",
                                "content": {
                                    "application/json": {
                                        "schema": {
                                            "$ref": "#/components/schemas/tests__test_pydantic_v1_v2_multifile__modelsv2__Item"
                                        }
                                    }
                                },
                            },
                            "422": {
                                "description": "Validation Error",
                                "content": {
                                    "application/json": {
                                        "schema": {
                                            "$ref": "#/components/schemas/HTTPValidationError"
                                        }
                                    }
                                },
                            },
                        },
                    }
                },
                "/v2-to-v1/item-to-list": {
                    "post": {
                        "summary": "Handle V2 Item To V1 List",
                        "operationId": "handle_v2_item_to_v1_list_v2_to_v1_item_to_list_post",
                        "requestBody": {
                            "content": {
                                "application/json": {
                                    "schema": pydantic_snapshot(
                                        v2=snapshot(
                                            {
                                                "$ref": "#/components/schemas/tests__test_pydantic_v1_v2_multifile__modelsv2__Item-Input"
                                            }
                                        ),
                                        v1=snapshot(
                                            {
                                                "$ref": "#/components/schemas/tests__test_pydantic_v1_v2_multifile__modelsv2__Item"
                                            }
                                        ),
                                    ),
                                }
                            },
                            "required": True,
                        },
                        "responses": {
                            "200": {
                                "description": "Successful Response",
                                "content": {
                                    "application/json": {
                                        "schema": {
                                            "items": {
                                                "$ref": "#/components/schemas/tests__test_pydantic_v1_v2_multifile__modelsv1__Item"
                                            },
                                            "type": "array",
                                            "title": "Response Handle V2 Item To V1 List V2 To V1 Item To List Post",
                                        }
                                    }
                                },
                            },
                            "422": {
                                "description": "Validation Error",
                                "content": {
                                    "application/json": {
                                        "schema": {
                                            "$ref": "#/components/schemas/HTTPValidationError"
                                        }
                                    }
                                },
                            },
                        },
                    }
                },
                "/v2-to-v1/list-to-list": {
                    "post": {
                        "summary": "Handle V2 List To V1 List",
                        "operationId": "handle_v2_list_to_v1_list_v2_to_v1_list_to_list_post",
                        "requestBody": {
                            "content": {
                                "application/json": {
                                    "schema": {
                                        "items": pydantic_snapshot(
                                            v2=snapshot(
                                                {
                                                    "$ref": "#/components/schemas/tests__test_pydantic_v1_v2_multifile__modelsv2__Item-Input"
                                                }
                                            ),
                                            v1=snapshot(
                                                {
                                                    "$ref": "#/components/schemas/tests__test_pydantic_v1_v2_multifile__modelsv2__Item"
                                                }
                                            ),
                                        ),
                                        "type": "array",
                                        "title": "Data",
                                    }
                                }
                            },
                            "required": True,
                        },
                        "responses": {
                            "200": {
                                "description": "Successful Response",
                                "content": {
                                    "application/json": {
                                        "schema": {
                                            "items": {
                                                "$ref": "#/components/schemas/tests__test_pydantic_v1_v2_multifile__modelsv1__Item"
                                            },
                                            "type": "array",
                                            "title": "Response Handle V2 List To V1 List V2 To V1 List To List Post",
                                        }
                                    }
                                },
                            },
                            "422": {
                                "description": "Validation Error",
                                "content": {
                                    "application/json": {
                                        "schema": {
                                            "$ref": "#/components/schemas/HTTPValidationError"
                                        }
                                    }
                                },
                            },
                        },
                    }
                },
                "/v2-to-v1/list-to-item": {
                    "post": {
                        "summary": "Handle V2 List To V1 Item",
                        "operationId": "handle_v2_list_to_v1_item_v2_to_v1_list_to_item_post",
                        "requestBody": {
                            "content": {
                                "application/json": {
                                    "schema": {
                                        "items": pydantic_snapshot(
                                            v2=snapshot(
                                                {
                                                    "$ref": "#/components/schemas/tests__test_pydantic_v1_v2_multifile__modelsv2__Item-Input"
                                                }
                                            ),
                                            v1=snapshot(
                                                {
                                                    "$ref": "#/components/schemas/tests__test_pydantic_v1_v2_multifile__modelsv2__Item"
                                                }
                                            ),
                                        ),
                                        "type": "array",
                                        "title": "Data",
                                    }
                                }
                            },
                            "required": True,
                        },
                        "responses": {
                            "200": {
                                "description": "Successful Response",
                                "content": {
                                    "application/json": {
                                        "schema": {
                                            "$ref": "#/components/schemas/tests__test_pydantic_v1_v2_multifile__modelsv1__Item"
                                        }
                                    }
                                },
                            },
                            "422": {
                                "description": "Validation Error",
                                "content": {
                                    "application/json": {
                                        "schema": {
                                            "$ref": "#/components/schemas/HTTPValidationError"
                                        }
                                    }
                                },
                            },
                        },
                    }
                },
                "/v2-to-v1/same-name": {
                    "post": {
                        "summary": "Handle V2 Same Name To V1",
                        "operationId": "handle_v2_same_name_to_v1_v2_to_v1_same_name_post",
                        "requestBody": {
                            "content": {
                                "application/json": {
                                    "schema": {
                                        "$ref": "#/components/schemas/Body_handle_v2_same_name_to_v1_v2_to_v1_same_name_post"
                                    }
                                }
                            },
                            "required": True,
                        },
                        "responses": {
                            "200": {
                                "description": "Successful Response",
                                "content": {
                                    "application/json": {
                                        "schema": {
                                            "$ref": "#/components/schemas/tests__test_pydantic_v1_v2_multifile__modelsv1__Item"
                                        }
                                    }
                                },
                            },
                            "422": {
                                "description": "Validation Error",
                                "content": {
                                    "application/json": {
                                        "schema": {
                                            "$ref": "#/components/schemas/HTTPValidationError"
                                        }
                                    }
                                },
                            },
                        },
                    }
                },
                "/v2-to-v1/list-of-items-to-list-of-items": {
                    "post": {
                        "summary": "Handle V2 Items In List To V1 Item In List",
                        "operationId": "handle_v2_items_in_list_to_v1_item_in_list_v2_to_v1_list_of_items_to_list_of_items_post",
                        "requestBody": {
                            "content": {
                                "application/json": {
                                    "schema": {
                                        "$ref": "#/components/schemas/Body_handle_v2_items_in_list_to_v1_item_in_list_v2_to_v1_list_of_items_to_list_of_items_post"
                                    }
                                }
                            },
                            "required": True,
                        },
                        "responses": {
                            "200": {
                                "description": "Successful Response",
                                "content": {
                                    "application/json": {
                                        "schema": {
                                            "items": {
                                                "$ref": "#/components/schemas/tests__test_pydantic_v1_v2_multifile__modelsv1__ItemInList"
                                            },
                                            "type": "array",
                                            "title": "Response Handle V2 Items In List To V1 Item In List V2 To V1 List Of Items To List Of Items Post",
                                        }
                                    }
                                },
                            },
                            "422": {
                                "description": "Validation Error",
                                "content": {
                                    "application/json": {
                                        "schema": {
                                            "$ref": "#/components/schemas/HTTPValidationError"
                                        }
                                    }
                                },
                            },
                        },
                    }
                },
            },
            "components": {
                "schemas": pydantic_snapshot(
                    v1=snapshot(
                        {
                            "Body_handle_v2_items_in_list_to_v1_item_in_list_v2_to_v1_list_of_items_to_list_of_items_post": {
                                "properties": {
                                    "data1": {
                                        "items": {
                                            "$ref": "#/components/schemas/tests__test_pydantic_v1_v2_multifile__modelsv2__ItemInList"
                                        },
                                        "type": "array",
                                        "title": "Data1",
                                    },
                                    "data2": {
                                        "items": {
                                            "$ref": "#/components/schemas/tests__test_pydantic_v1_v2_multifile__modelsv2b__ItemInList"
                                        },
                                        "type": "array",
                                        "title": "Data2",
                                    },
                                },
                                "type": "object",
                                "required": ["data1", "data2"],
                                "title": "Body_handle_v2_items_in_list_to_v1_item_in_list_v2_to_v1_list_of_items_to_list_of_items_post",
                            },
                            "Body_handle_v2_same_name_to_v1_v2_to_v1_same_name_post": {
                                "properties": {
                                    "item1": {
                                        "$ref": "#/components/schemas/tests__test_pydantic_v1_v2_multifile__modelsv2__Item"
                                    },
                                    "item2": {
                                        "$ref": "#/components/schemas/tests__test_pydantic_v1_v2_multifile__modelsv2b__Item"
                                    },
                                },
                                "type": "object",
                                "required": ["item1", "item2"],
                                "title": "Body_handle_v2_same_name_to_v1_v2_to_v1_same_name_post",
                            },
                            "HTTPValidationError": {
                                "properties": {
                                    "detail": {
                                        "items": {
                                            "$ref": "#/components/schemas/ValidationError"
                                        },
                                        "type": "array",
                                        "title": "Detail",
                                    }
                                },
                                "type": "object",
                                "title": "HTTPValidationError",
                            },
                            "ValidationError": {
                                "properties": {
                                    "loc": {
                                        "items": {
                                            "anyOf": [
                                                {"type": "string"},
                                                {"type": "integer"},
                                            ]
                                        },
                                        "type": "array",
                                        "title": "Location",
                                    },
                                    "msg": {"type": "string", "title": "Message"},
                                    "type": {"type": "string", "title": "Error Type"},
                                },
                                "type": "object",
                                "required": ["loc", "msg", "type"],
                                "title": "ValidationError",
                            },
                            "tests__test_pydantic_v1_v2_multifile__modelsv1__Item": {
                                "properties": {
                                    "title": {"type": "string", "title": "Title"},
                                    "size": {"type": "integer", "title": "Size"},
                                    "description": {
                                        "type": "string",
                                        "title": "Description",
                                    },
                                    "sub": {
                                        "$ref": "#/components/schemas/tests__test_pydantic_v1_v2_multifile__modelsv1__SubItem"
                                    },
                                    "multi": {
                                        "items": {
                                            "$ref": "#/components/schemas/tests__test_pydantic_v1_v2_multifile__modelsv1__SubItem"
                                        },
                                        "type": "array",
                                        "title": "Multi",
                                        "default": [],
                                    },
                                },
                                "type": "object",
                                "required": ["title", "size", "sub"],
                                "title": "Item",
                            },
                            "tests__test_pydantic_v1_v2_multifile__modelsv1__ItemInList": {
                                "properties": {
                                    "name1": {"type": "string", "title": "Name1"}
                                },
                                "type": "object",
                                "required": ["name1"],
                                "title": "ItemInList",
                            },
                            "tests__test_pydantic_v1_v2_multifile__modelsv1__SubItem": {
                                "properties": {
                                    "name": {"type": "string", "title": "Name"}
                                },
                                "type": "object",
                                "required": ["name"],
                                "title": "SubItem",
                            },
                            "tests__test_pydantic_v1_v2_multifile__modelsv2__Item": {
                                "properties": {
                                    "new_title": {
                                        "type": "string",
                                        "title": "New Title",
                                    },
                                    "new_size": {
                                        "type": "integer",
                                        "title": "New Size",
                                    },
                                    "new_description": {
                                        "type": "string",
                                        "title": "New Description",
                                    },
                                    "new_sub": {
                                        "$ref": "#/components/schemas/tests__test_pydantic_v1_v2_multifile__modelsv2__SubItem"
                                    },
                                    "new_multi": {
                                        "items": {
                                            "$ref": "#/components/schemas/tests__test_pydantic_v1_v2_multifile__modelsv2__SubItem"
                                        },
                                        "type": "array",
                                        "title": "New Multi",
                                        "default": [],
                                    },
                                },
                                "type": "object",
                                "required": ["new_title", "new_size", "new_sub"],
                                "title": "Item",
                            },
                            "tests__test_pydantic_v1_v2_multifile__modelsv2__ItemInList": {
                                "properties": {
                                    "name2": {"type": "string", "title": "Name2"}
                                },
                                "type": "object",
                                "required": ["name2"],
                                "title": "ItemInList",
                            },
                            "tests__test_pydantic_v1_v2_multifile__modelsv2__SubItem": {
                                "properties": {
                                    "new_sub_name": {
                                        "type": "string",
                                        "title": "New Sub Name",
                                    }
                                },
                                "type": "object",
                                "required": ["new_sub_name"],
                                "title": "SubItem",
                            },
                            "tests__test_pydantic_v1_v2_multifile__modelsv2b__Item": {
                                "properties": {
                                    "dup_title": {
                                        "type": "string",
                                        "title": "Dup Title",
                                    },
                                    "dup_size": {
                                        "type": "integer",
                                        "title": "Dup Size",
                                    },
                                    "dup_description": {
                                        "type": "string",
                                        "title": "Dup Description",
                                    },
                                    "dup_sub": {
                                        "$ref": "#/components/schemas/tests__test_pydantic_v1_v2_multifile__modelsv2b__SubItem"
                                    },
                                    "dup_multi": {
                                        "items": {
                                            "$ref": "#/components/schemas/tests__test_pydantic_v1_v2_multifile__modelsv2b__SubItem"
                                        },
                                        "type": "array",
                                        "title": "Dup Multi",
                                        "default": [],
                                    },
                                },
                                "type": "object",
                                "required": ["dup_title", "dup_size", "dup_sub"],
                                "title": "Item",
                            },
                            "tests__test_pydantic_v1_v2_multifile__modelsv2b__ItemInList": {
                                "properties": {
                                    "dup_name2": {
                                        "type": "string",
                                        "title": "Dup Name2",
                                    }
                                },
                                "type": "object",
                                "required": ["dup_name2"],
                                "title": "ItemInList",
                            },
                            "tests__test_pydantic_v1_v2_multifile__modelsv2b__SubItem": {
                                "properties": {
                                    "dup_sub_name": {
                                        "type": "string",
                                        "title": "Dup Sub Name",
                                    }
                                },
                                "type": "object",
                                "required": ["dup_sub_name"],
                                "title": "SubItem",
                            },
                        }
                    ),
                    v2=snapshot(
                        {
                            "Body_handle_v2_items_in_list_to_v1_item_in_list_v2_to_v1_list_of_items_to_list_of_items_post": {
                                "properties": {
                                    "data1": {
                                        "items": {
                                            "$ref": "#/components/schemas/tests__test_pydantic_v1_v2_multifile__modelsv2__ItemInList"
                                        },
                                        "type": "array",
                                        "title": "Data1",
                                    },
                                    "data2": {
                                        "items": {
                                            "$ref": "#/components/schemas/tests__test_pydantic_v1_v2_multifile__modelsv2b__ItemInList"
                                        },
                                        "type": "array",
                                        "title": "Data2",
                                    },
                                },
                                "type": "object",
                                "required": ["data1", "data2"],
                                "title": "Body_handle_v2_items_in_list_to_v1_item_in_list_v2_to_v1_list_of_items_to_list_of_items_post",
                            },
                            "Body_handle_v2_same_name_to_v1_v2_to_v1_same_name_post": {
                                "properties": {
                                    "item1": {
                                        "$ref": "#/components/schemas/tests__test_pydantic_v1_v2_multifile__modelsv2__Item-Input"
                                    },
                                    "item2": {
                                        "$ref": "#/components/schemas/tests__test_pydantic_v1_v2_multifile__modelsv2b__Item"
                                    },
                                },
                                "type": "object",
                                "required": ["item1", "item2"],
                                "title": "Body_handle_v2_same_name_to_v1_v2_to_v1_same_name_post",
                            },
                            "HTTPValidationError": {
                                "properties": {
                                    "detail": {
                                        "items": {
                                            "$ref": "#/components/schemas/ValidationError"
                                        },
                                        "type": "array",
                                        "title": "Detail",
                                    }
                                },
                                "type": "object",
                                "title": "HTTPValidationError",
                            },
                            "ValidationError": {
                                "properties": {
                                    "loc": {
                                        "items": {
                                            "anyOf": [
                                                {"type": "string"},
                                                {"type": "integer"},
                                            ]
                                        },
                                        "type": "array",
                                        "title": "Location",
                                    },
                                    "msg": {"type": "string", "title": "Message"},
                                    "type": {"type": "string", "title": "Error Type"},
                                },
                                "type": "object",
                                "required": ["loc", "msg", "type"],
                                "title": "ValidationError",
                            },
                            "tests__test_pydantic_v1_v2_multifile__modelsv1__Item": {
                                "properties": {
                                    "title": {"type": "string", "title": "Title"},
                                    "size": {"type": "integer", "title": "Size"},
                                    "description": {
                                        "type": "string",
                                        "title": "Description",
                                    },
                                    "sub": {
                                        "$ref": "#/components/schemas/tests__test_pydantic_v1_v2_multifile__modelsv1__SubItem"
                                    },
                                    "multi": {
                                        "items": {
                                            "$ref": "#/components/schemas/tests__test_pydantic_v1_v2_multifile__modelsv1__SubItem"
                                        },
                                        "type": "array",
                                        "title": "Multi",
                                        "default": [],
                                    },
                                },
                                "type": "object",
                                "required": ["title", "size", "sub"],
                                "title": "Item",
                            },
                            "tests__test_pydantic_v1_v2_multifile__modelsv1__ItemInList": {
                                "properties": {
                                    "name1": {"type": "string", "title": "Name1"}
                                },
                                "type": "object",
                                "required": ["name1"],
                                "title": "ItemInList",
                            },
                            "tests__test_pydantic_v1_v2_multifile__modelsv1__SubItem": {
                                "properties": {
                                    "name": {"type": "string", "title": "Name"}
                                },
                                "type": "object",
                                "required": ["name"],
                                "title": "SubItem",
                            },
                            "tests__test_pydantic_v1_v2_multifile__modelsv2__Item": {
                                "properties": {
                                    "new_title": {
                                        "type": "string",
                                        "title": "New Title",
                                    },
                                    "new_size": {
                                        "type": "integer",
                                        "title": "New Size",
                                    },
                                    "new_description": {
                                        "anyOf": [{"type": "string"}, {"type": "null"}],
                                        "title": "New Description",
                                    },
                                    "new_sub": {
                                        "$ref": "#/components/schemas/tests__test_pydantic_v1_v2_multifile__modelsv2__SubItem"
                                    },
                                    "new_multi": {
                                        "items": {
                                            "$ref": "#/components/schemas/tests__test_pydantic_v1_v2_multifile__modelsv2__SubItem"
                                        },
                                        "type": "array",
                                        "title": "New Multi",
                                        "default": [],
                                    },
                                },
                                "type": "object",
                                "required": ["new_title", "new_size", "new_sub"],
                                "title": "Item",
                            },
                            "tests__test_pydantic_v1_v2_multifile__modelsv2__Item-Input": {
                                "properties": {
                                    "new_title": {
                                        "type": "string",
                                        "title": "New Title",
                                    },
                                    "new_size": {
                                        "type": "integer",
                                        "title": "New Size",
                                    },
                                    "new_description": {
                                        "anyOf": [{"type": "string"}, {"type": "null"}],
                                        "title": "New Description",
                                    },
                                    "new_sub": {
                                        "$ref": "#/components/schemas/tests__test_pydantic_v1_v2_multifile__modelsv2__SubItem"
                                    },
                                    "new_multi": {
                                        "items": {
                                            "$ref": "#/components/schemas/tests__test_pydantic_v1_v2_multifile__modelsv2__SubItem"
                                        },
                                        "type": "array",
                                        "title": "New Multi",
                                        "default": [],
                                    },
                                },
                                "type": "object",
                                "required": ["new_title", "new_size", "new_sub"],
                                "title": "Item",
                            },
                            "tests__test_pydantic_v1_v2_multifile__modelsv2__ItemInList": {
                                "properties": {
                                    "name2": {"type": "string", "title": "Name2"}
                                },
                                "type": "object",
                                "required": ["name2"],
                                "title": "ItemInList",
                            },
                            "tests__test_pydantic_v1_v2_multifile__modelsv2__SubItem": {
                                "properties": {
                                    "new_sub_name": {
                                        "type": "string",
                                        "title": "New Sub Name",
                                    }
                                },
                                "type": "object",
                                "required": ["new_sub_name"],
                                "title": "SubItem",
                            },
                            "tests__test_pydantic_v1_v2_multifile__modelsv2b__Item": {
                                "properties": {
                                    "dup_title": {
                                        "type": "string",
                                        "title": "Dup Title",
                                    },
                                    "dup_size": {
                                        "type": "integer",
                                        "title": "Dup Size",
                                    },
                                    "dup_description": {
                                        "anyOf": [{"type": "string"}, {"type": "null"}],
                                        "title": "Dup Description",
                                    },
                                    "dup_sub": {
                                        "$ref": "#/components/schemas/tests__test_pydantic_v1_v2_multifile__modelsv2b__SubItem"
                                    },
                                    "dup_multi": {
                                        "items": {
                                            "$ref": "#/components/schemas/tests__test_pydantic_v1_v2_multifile__modelsv2b__SubItem"
                                        },
                                        "type": "array",
                                        "title": "Dup Multi",
                                        "default": [],
                                    },
                                },
                                "type": "object",
                                "required": ["dup_title", "dup_size", "dup_sub"],
                                "title": "Item",
                            },
                            "tests__test_pydantic_v1_v2_multifile__modelsv2b__ItemInList": {
                                "properties": {
                                    "dup_name2": {
                                        "type": "string",
                                        "title": "Dup Name2",
                                    }
                                },
                                "type": "object",
                                "required": ["dup_name2"],
                                "title": "ItemInList",
                            },
                            "tests__test_pydantic_v1_v2_multifile__modelsv2b__SubItem": {
                                "properties": {
                                    "dup_sub_name": {
                                        "type": "string",
                                        "title": "Dup Sub Name",
                                    }
                                },
                                "type": "object",
                                "required": ["dup_sub_name"],
                                "title": "SubItem",
                            },
                        }
                    ),
                ),
            },
        }
    )



from fastapi import FastAPI
from fastapi.responses import JSONResponse
from pydantic import BaseModel


class Item(BaseModel):
    id: str
    value: str


class Message(BaseModel):
    message: str


app = FastAPI()


@app.get(
    "/items/{item_id}",
    response_model=Item,
    responses={
        404: {"model": Message, "description": "The item was not found"},
        200: {
            "description": "Item requested by ID",
            "content": {
                "application/json": {
                    "example": {"id": "bar", "value": "The bar tenders"}
                }
            },
        },
    },
)
async def read_item(item_id: str):
    if item_id == "foo":
        return {"id": "foo", "value": "there goes my hero"}
    else:
        return JSONResponse(status_code=404, content={"message": "Item not found"})



from typing import Dict

import pytest
from fastapi import Depends, FastAPI, Security
from fastapi.testclient import TestClient
from typing_extensions import Annotated


@pytest.fixture(name="call_counter")
def call_counter_fixture():
    return {"count": 0}


@pytest.fixture(name="app")
def app_fixture(call_counter: Dict[str, int]):
    def get_db():
        call_counter["count"] += 1
        return f"db_{call_counter['count']}"

    def get_user(db: Annotated[str, Depends(get_db)]):
        return "user"

    app = FastAPI()

    @app.get("/")
    def endpoint(
        db: Annotated[str, Depends(get_db)],
        user: Annotated[str, Security(get_user, scopes=["read"])],
    ):
        return {"db": db}

    return app


@pytest.fixture(name="client")
def client_fixture(app: FastAPI):
    return TestClient(app)


def test_security_scopes_dependency_called_once(
    client: TestClient, call_counter: Dict[str, int]
):
    response = client.get("/")

    assert response.status_code == 200
    assert call_counter["count"] == 1



import time

from fastapi import FastAPI, Request

app = FastAPI()


@app.middleware("http")
async def add_process_time_header(request: Request, call_next):
    start_time = time.perf_counter()
    response = await call_next(request)
    process_time = time.perf_counter() - start_time
    response.headers["X-Process-Time"] = str(process_time)
    return response



from typing import Union

from fastapi import FastAPI, Form
from fastapi.testclient import TestClient
from pydantic import BaseModel
from typing_extensions import Annotated

app = FastAPI()


class UserForm(BaseModel):
    name: str
    email: str


class CompanyForm(BaseModel):
    company_name: str
    industry: str


@app.post("/form-union/")
def post_union_form(data: Annotated[Union[UserForm, CompanyForm], Form()]):
    return {"received": data}


client = TestClient(app)


def test_post_user_form():
    response = client.post(
        "/form-union/", data={"name": "John Doe", "email": "john@example.com"}
    )
    assert response.status_code == 200, response.text
    assert response.json() == {
        "received": {"name": "John Doe", "email": "john@example.com"}
    }


def test_post_company_form():
    response = client.post(
        "/form-union/", data={"company_name": "Tech Corp", "industry": "Technology"}
    )
    assert response.status_code == 200, response.text
    assert response.json() == {
        "received": {"company_name": "Tech Corp", "industry": "Technology"}
    }


def test_invalid_form_data():
    response = client.post(
        "/form-union/",
        data={"name": "John", "company_name": "Tech Corp"},
    )
    assert response.status_code == 422, response.text


def test_empty_form():
    response = client.post("/form-union/")
    assert response.status_code == 422, response.text


def test_openapi_schema():
    response = client.get("/openapi.json")
    assert response.status_code == 200, response.text

    assert response.json() == {
        "openapi": "3.1.0",
        "info": {"title": "FastAPI", "version": "0.1.0"},
        "paths": {
            "/form-union/": {
                "post": {
                    "summary": "Post Union Form",
                    "operationId": "post_union_form_form_union__post",
                    "requestBody": {
                        "content": {
                            "application/x-www-form-urlencoded": {
                                "schema": {
                                    "anyOf": [
                                        {"$ref": "#/components/schemas/UserForm"},
                                        {"$ref": "#/components/schemas/CompanyForm"},
                                    ],
                                    "title": "Data",
                                }
                            }
                        },
                        "required": True,
                    },
                    "responses": {
                        "200": {
                            "description": "Successful Response",
                            "content": {"application/json": {"schema": {}}},
                        },
                        "422": {
                            "description": "Validation Error",
                            "content": {
                                "application/json": {
                                    "schema": {
                                        "$ref": "#/components/schemas/HTTPValidationError"
                                    }
                                }
                            },
                        },
                    },
                }
            }
        },
        "components": {
            "schemas": {
                "CompanyForm": {
                    "properties": {
                        "company_name": {"type": "string", "title": "Company Name"},
                        "industry": {"type": "string", "title": "Industry"},
                    },
                    "type": "object",
                    "required": ["company_name", "industry"],
                    "title": "CompanyForm",
                },
                "HTTPValidationError": {
                    "properties": {
                        "detail": {
                            "items": {"$ref": "#/components/schemas/ValidationError"},
                            "type": "array",
                            "title": "Detail",
                        }
                    },
                    "type": "object",
                    "title": "HTTPValidationError",
                },
                "UserForm": {
                    "properties": {
                        "name": {"type": "string", "title": "Name"},
                        "email": {"type": "string", "title": "Email"},
                    },
                    "type": "object",
                    "required": ["name", "email"],
                    "title": "UserForm",
                },
                "ValidationError": {
                    "properties": {
                        "loc": {
                            "items": {
                                "anyOf": [{"type": "string"}, {"type": "integer"}]
                            },
                            "type": "array",
                            "title": "Location",
                        },
                        "msg": {"type": "string", "title": "Message"},
                        "type": {"type": "string", "title": "Error Type"},
                    },
                    "type": "object",
                    "required": ["loc", "msg", "type"],
                    "title": "ValidationError",
                },
            }
        },
    }



import importlib
from types import ModuleType

import pytest
from dirty_equals import IsDict
from fastapi.testclient import TestClient

from tests.utils import needs_py310


@pytest.fixture(
    name="mod",
    params=[
        pytest.param("tutorial001"),
        pytest.param("tutorial001_py310", marks=needs_py310),
    ],
)
def get_mod(request: pytest.FixtureRequest):
    mod = importlib.import_module(f"docs_src.openapi_callbacks.{request.param}")
    return mod


@pytest.fixture(name="client")
def get_client(mod: ModuleType):
    client = TestClient(mod.app)
    client.headers.clear()
    return client


def test_get(client: TestClient):
    response = client.post(
        "/invoices/", json={"id": "fooinvoice", "customer": "John", "total": 5.3}
    )
    assert response.status_code == 200, response.text
    assert response.json() == {"msg": "Invoice received"}


def test_dummy_callback(mod: ModuleType):
    # Just for coverage
    mod.invoice_notification({})


def test_openapi_schema(client: TestClient):
    response = client.get("/openapi.json")
    assert response.status_code == 200, response.text
    assert response.json() == {
        "openapi": "3.1.0",
        "info": {"title": "FastAPI", "version": "0.1.0"},
        "paths": {
            "/invoices/": {
                "post": {
                    "summary": "Create Invoice",
                    "description": 'Create an invoice.\n\nThis will (let\'s imagine) let the API user (some external developer) create an\ninvoice.\n\nAnd this path operation will:\n\n* Send the invoice to the client.\n* Collect the money from the client.\n* Send a notification back to the API user (the external developer), as a callback.\n    * At this point is that the API will somehow send a POST request to the\n        external API with the notification of the invoice event\n        (e.g. "payment successful").',
                    "operationId": "create_invoice_invoices__post",
                    "parameters": [
                        {
                            "required": False,
                            "schema": IsDict(
                                {
                                    "anyOf": [
                                        {
                                            "type": "string",
                                            "format": "uri",
                                            "minLength": 1,
                                            "maxLength": 2083,
                                        },
                                        {"type": "null"},
                                    ],
                                    "title": "Callback Url",
                                }
                            )
                            | IsDict(
                                # TODO: remove when deprecating Pydantic v1
                                {
                                    "title": "Callback Url",
                                    "maxLength": 2083,
                                    "minLength": 1,
                                    "type": "string",
                                    "format": "uri",
                                }
                            ),
                            "name": "callback_url",
                            "in": "query",
                        }
                    ],
                    "requestBody": {
                        "content": {
                            "application/json": {
                                "schema": {"$ref": "#/components/schemas/Invoice"}
                            }
                        },
                        "required": True,
                    },
                    "responses": {
                        "200": {
                            "description": "Successful Response",
                            "content": {"application/json": {"schema": {}}},
                        },
                        "422": {
                            "description": "Validation Error",
                            "content": {
                                "application/json": {
                                    "schema": {
                                        "$ref": "#/components/schemas/HTTPValidationError"
                                    }
                                }
                            },
                        },
                    },
                    "callbacks": {
                        "invoice_notification": {
                            "{$callback_url}/invoices/{$request.body.id}": {
                                "post": {
                                    "summary": "Invoice Notification",
                                    "operationId": "invoice_notification__callback_url__invoices___request_body_id__post",
                                    "requestBody": {
                                        "required": True,
                                        "content": {
                                            "application/json": {
                                                "schema": {
                                                    "$ref": "#/components/schemas/InvoiceEvent"
                                                }
                                            }
                                        },
                                    },
                                    "responses": {
                                        "200": {
                                            "description": "Successful Response",
                                            "content": {
                                                "application/json": {
                                                    "schema": {
                                                        "$ref": "#/components/schemas/InvoiceEventReceived"
                                                    }
                                                }
                                            },
                                        },
                                        "422": {
                                            "description": "Validation Error",
                                            "content": {
                                                "application/json": {
                                                    "schema": {
                                                        "$ref": "#/components/schemas/HTTPValidationError"
                                                    }
                                                }
                                            },
                                        },
                                    },
                                }
                            }
                        }
                    },
                }
            }
        },
        "components": {
            "schemas": {
                "HTTPValidationError": {
                    "title": "HTTPValidationError",
                    "type": "object",
                    "properties": {
                        "detail": {
                            "title": "Detail",
                            "type": "array",
                            "items": {"$ref": "#/components/schemas/ValidationError"},
                        }
                    },
                },
                "Invoice": {
                    "title": "Invoice",
                    "required": ["id", "customer", "total"],
                    "type": "object",
                    "properties": {
                        "id": {"title": "Id", "type": "string"},
                        "title": IsDict(
                            {
                                "title": "Title",
                                "anyOf": [{"type": "string"}, {"type": "null"}],
                            }
                        )
                        | IsDict(
                            # TODO: remove when deprecating Pydantic v1
                            {"title": "Title", "type": "string"}
                        ),
                        "customer": {"title": "Customer", "type": "string"},
                        "total": {"title": "Total", "type": "number"},
                    },
                },
                "InvoiceEvent": {
                    "title": "InvoiceEvent",
                    "required": ["description", "paid"],
                    "type": "object",
                    "properties": {
                        "description": {"title": "Description", "type": "string"},
                        "paid": {"title": "Paid", "type": "boolean"},
                    },
                },
                "InvoiceEventReceived": {
                    "title": "InvoiceEventReceived",
                    "required": ["ok"],
                    "type": "object",
                    "properties": {"ok": {"title": "Ok", "type": "boolean"}},
                },
                "ValidationError": {
                    "title": "ValidationError",
                    "required": ["loc", "msg", "type"],
                    "type": "object",
                    "properties": {
                        "loc": {
                            "title": "Location",
                            "type": "array",
                            "items": {
                                "anyOf": [{"type": "string"}, {"type": "integer"}]
                            },
                        },
                        "msg": {"title": "Message", "type": "string"},
                        "type": {"title": "Error Type", "type": "string"},
                    },
                },
            }
        },
    }



from fastapi import FastAPI
from fastapi.testclient import TestClient
from pydantic import BaseModel, Field

app = FastAPI()


class ResponseModel(BaseModel):
    code: int = 200
    message: str = Field(default_factory=lambda: "Successful operation.")


@app.get(
    "/response_model_has_default_factory_return_dict",
    response_model=ResponseModel,
)
async def response_model_has_default_factory_return_dict():
    return {"code": 200}


@app.get(
    "/response_model_has_default_factory_return_model",
    response_model=ResponseModel,
)
async def response_model_has_default_factory_return_model():
    return ResponseModel()


client = TestClient(app)


def test_response_model_has_default_factory_return_dict():
    response = client.get("/response_model_has_default_factory_return_dict")

    assert response.status_code == 200, response.text

    assert response.json()["code"] == 200
    assert response.json()["message"] == "Successful operation."


def test_response_model_has_default_factory_return_model():
    response = client.get("/response_model_has_default_factory_return_model")

    assert response.status_code == 200, response.text

    assert response.json()["code"] == 200
    assert response.json()["message"] == "Successful operation."



from fastapi import FastAPI
from fastapi.responses import FileResponse
from pydantic import BaseModel


class Item(BaseModel):
    id: str
    value: str


app = FastAPI()


@app.get(
    "/items/{item_id}",
    response_model=Item,
    responses={
        200: {
            "content": {"image/png": {}},
            "description": "Return the JSON item or an image.",
        }
    },
)
async def read_item(item_id: str, img: bool | None = None):
    if img:
        return FileResponse("image.png", media_type="image/png")
    else:
        return {"id": "foo", "value": "there goes my hero"}
